
package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.Conquistadores.App.R;

/**
 * Clase que lista las conquistas y duelos sufridos por el equipo en un periodo de tiempo.
 *
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class EnfrentamientoListar extends ListActivity implements Runnable{
	
	static private String TAG = "EnfrentamientoListar";	
	
	private final int MENU_AYUDA = Menu.FIRST;
	private final int RESULTADO_ERROR_CONEXION = -2;
	private final int RESULTADO_ERROR = -1;
	private final int RESULTADO_OK = 1;
		 
	static private List<String> listInformeEnfrentamiento;
	static private List<String> listIcono;
	static private List<Boolean> listVictoria;
	
	private String conquistaApliacionId;
	private String dueloAplicacionId;
	
	private ProgressDialog progressDialog;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Bundle extras = getIntent().getExtras();
		conquistaApliacionId = extras.getString("conquistaAplicacionId");
		dueloAplicacionId = extras.getString("dueloAplicacionId");		
		
		listInformeEnfrentamiento = new ArrayList<String>();
		listIcono = new ArrayList<String>();
		listVictoria = new ArrayList<Boolean>();
		
		progressDialog = ProgressDialog.show(this, "", 
				getString(R.string.progDialogEnfrentamiento), true);      
		Thread thread = new Thread(this);
		thread.start();
	}
	
	@Override
	protected void onPause(){
		super.onPause();
		finish();
	}

    @Override
	public void run() {
		Looper.prepare();
		obtenerEnfrentamientos();
		Looper.myLooper().quit();
		Looper.loop();
	}

	private Handler handler = new Handler(){
		@Override
    	public void handleMessage(Message msg){
			int valor = msg.what;
			int icono;
			String titulo;
			String msgAlert;
			
			progressDialog.cancel();
    		switch(valor){
			case RESULTADO_ERROR_CONEXION:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloConexionServer);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_ERROR:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloDatosRecibidos);
				showAlert(icono,titulo,msgAlert);
			break;
       		case RESULTADO_OK:
       			EventsAdapter adapter = new EventsAdapter(EnfrentamientoListar.this);
				setListAdapter(adapter);
				int dueloServidorId = ServicioEnfrentamiento.getDueloServidorId();
				ServicioEnfrentamiento.setDueloAplicacionId(dueloServidorId);		
				int conquistaServidorId = ServicioEnfrentamiento.getConquistaServidorId();
				ServicioEnfrentamiento.setConquistaAplicacionId(conquistaServidorId);
    			break;
    		};
    	}
	};
	
	private void obtenerEnfrentamientos(){
		int resultado;		
		try{
			String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
			int idPartida = PartidaListar.getPartida().getIdent();
			int idEquipo = RecursoListar.getEquipo().getIdent();
			String url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/conquista/%4$s/duelo/%5$s/listar/?format=json",
					urlServer,idPartida,idEquipo, conquistaApliacionId, dueloAplicacionId);
			String str = Utilidades.obtenerDatos(url);
			resultado = procesarRespuestaEnfrentamientoListar(str);
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }catch (IOException e) {
		    Log.e(TAG, e.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }
	}
    
    private int procesarRespuestaEnfrentamientoListar(String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;
			
			listInformeEnfrentamiento = new ArrayList<String>();
			listIcono = new ArrayList<String>();
			listVictoria = new ArrayList<Boolean>();
			
			if (json.has("conquistas") || json.has("duelos")){
				String equipoNombre;
				boolean victoriaAtacante;				
				String territorioNombre;

				JSONArray conquistasObject = json.getJSONArray("conquistas");
				int longitudConquistas = conquistasObject.length()-1;
					
				for (int i = 0; i < longitudConquistas; i++){
					JSONObject JSONConquista = (JSONObject)conquistasObject.get(i);
					
					equipoNombre = JSONConquista.getString("equipoNombre");
					territorioNombre = JSONConquista.getString("territorioNombre");
					victoriaAtacante = JSONConquista.getBoolean("victoria");
					
					listVictoria.add(victoriaAtacante);	
					listIcono.add("conquista");
					if (victoriaAtacante)
						listInformeEnfrentamiento.add(String.format(getString(R.string.ataqueRecibidoPerdido),
								equipoNombre, territorioNombre));
					else
						listInformeEnfrentamiento.add(String.format(getString(R.string.ataqueRecibidoGanado),
								equipoNombre, territorioNombre));
				}			
									
				JSONArray duelosObject = json.getJSONArray("duelos");
				int longitudDuelos = duelosObject.length()-1;
					
				for (int i = 0; i < longitudDuelos; i++){
					JSONObject JSONDuelo = (JSONObject)duelosObject.get(i);
					
					equipoNombre = JSONDuelo.getString("equipoNombre");
					victoriaAtacante = JSONDuelo.getBoolean("victoria");
					
					listVictoria.add(victoriaAtacante);
					listIcono.add("duelo");
					if (victoriaAtacante)
						listInformeEnfrentamiento.add(String.format(getString(R.string.dueloRecibidoPerdido),
								equipoNombre));
					else
						listInformeEnfrentamiento.add(String.format(getString(R.string.dueloRecibidoGanado),
								equipoNombre));
				}
			}else
				return RESULTADO_ERROR;

		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
        return RESULTADO_OK;
	}
    
	private void showAlert(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	        	   EnfrentamientoListar.this.setResult(RESULT_OK);
    	        	   EnfrentamientoListar.this.finish();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    }
    
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaEnfrentamientoListar);
    		showAlertAyuda(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
	private void showAlertAyuda(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    } 
	
    public static class EventsAdapter extends BaseAdapter {
    	
    	Context context;
    	
    	EventsAdapter(Context context) {
    		this.context = context;
    	}
    	
		public int getCount() {
			// TODO Auto-generated method stub
			return listInformeEnfrentamiento.size();
		}
    	
		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

    	public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) context
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.enfrentamiento_listar, null);
	        } else {
	            view = convertView;
	        } 	        
			
			LinearLayout linearLayout = (LinearLayout)view.findViewById(R.id.linearLayout1);
			if (listVictoria.get(position))
				linearLayout.setBackgroundDrawable(context.getResources().getDrawable(R.drawable.fondo_rojo));
			else
				linearLayout.setBackgroundDrawable(context.getResources().getDrawable(R.drawable.fondo_verde));
			
			TextView tipoEnfrentamiento = (TextView)view.findViewById(R.id.textTipoEnfrentamiento);
			ImageView icono = (ImageView)view.findViewById(R.id.iconoEnfrentamieto);
			if (listIcono.get(position).compareTo("conquista") == 0){
				icono.setImageDrawable(context.getResources().getDrawable(R.drawable.ic_map_conquistar));
				tipoEnfrentamiento.setText(context.getString(R.string.conquista));
			}else{
				icono.setImageDrawable(context.getResources().getDrawable(R.drawable.ic_map_duelo));	
				tipoEnfrentamiento.setText(context.getString(R.string.duelo));
			}
			TextView textoEnfrentamiento = (TextView)view.findViewById(R.id.textEnfrentamiento);
			textoEnfrentamiento.setText(listInformeEnfrentamiento.get(position));
			
			return view;
		}
    }
}
