package com.Conquistadores.App;

import java.util.ArrayList;
import java.util.List;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.drawable.Drawable;
import android.widget.ListAdapter;

import com.Conquistadores.App.R;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.OverlayItem;

/**
 * Clase auxiliar para aadir la capa de los equipos rivales al MapActivity de la aplicacin.
 * Mediante el mtodo draw, representamos los equipos rivales con iconos personalizados
 * segn el color de ellos.
 * Si el usuario pulsa sobre un equipo rival en el mapa, mediante el mtodo onTap, se 
 * muestra el correspondiente context menu.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class EquipoItemizedOverlay extends ItemizedOverlay<OverlayItem> {

	private List<OverlayItem> mOverlays = new ArrayList<OverlayItem>();
	private Context context;
	
	public EquipoItemizedOverlay(Context context, Drawable defaultMarker) {
		super(boundCenterBottom(defaultMarker));
		this.context = context;
	}

	@Override
	protected OverlayItem createItem(int i) {
	  return mOverlays.get(i);
	}
	
    @Override 
    protected boolean onTap(int indice){
    	final int equipoId = Integer.valueOf(mOverlays.get(indice).getSnippet());
    	String titulo = context.getString(R.string.menuEquipoRival);
    	final MenuFila[] opciones = new MenuFila[1];
    	opciones[0] = new MenuFila(R.drawable.ic_map_duelo, context.getString(R.string.menuDuelo));

    	ListAdapter adapter = new MenuAdapter(context, opciones);
    	AlertDialog.Builder builder = new AlertDialog.Builder(context);
    	builder.setAdapter(adapter, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int item) {
            	dialog.dismiss();
            	OpcionesMenuEquipo opcionMenu = new OpcionesMenuEquipo(context, equipoId);
                String opcion = opciones[item].getTitulo();				
	            if(opcion.compareTo(context.getString(R.string.menuDuelo)) == 0){					
					opcionMenu.duelo();	
				}
            }
        });

		AlertDialog alert = builder.create();
		alert.setTitle(titulo);
    	alert.show();
    	return super.onTap(indice);
    }

	@Override
	public int size() {
		return mOverlays.size();
	}
	
	public void addOverlay(OverlayItem overlay) {
	    mOverlays.add(overlay);
	    populate();
	}

}
