
package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.Conquistadores.App.R;

/**
 * Clase para obtener informacin de la partida en curso y mostrarla al usuario.
 * El jugador puede ver el tiempo restante de la partida as como informacin de su equipo.
 * Tambin puede observar la clasificacin de todos los equipos, ordenado de mayor a menor
 * puntuacin.
 * 
 * Cuando el tiempo de la partida acabe, se le mostrar un alert dialog al usuario
 * informndole de la situacin.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class Estadisticas extends Activity implements Runnable{
	
	static private String TAG = "Estadisticas";	
	
	private final int MENU_AYUDA = Menu.FIRST;
	private final int RESULTADO_ERROR_CONEXION = -2;
	private final int RESULTADO_ERROR = -1;
	private final int RESULTADO_OK = 1;
	
	//private boolean finThreadReloj = false;
	 
	/*Variables que representan la IU*/
	private LinearLayout linearLayout;
	private static TextView tiempoRestante;	
	private TextView miEquipo;
	private String infoEquipo;
	private ListView clasificacion; 
	
	static private List<String> listNombre;
	static private List<String> listColor;
	static private List<String> listNumTerritorios;
	static private List<String> listPuntuacion;
	
	private ProgressDialog progressDialog;
	
	public static TextView getTiempoRestante() {
		return tiempoRestante;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.partida_estadisticas);
		linearLayout = (LinearLayout)findViewById(R.id.LinearLayout1);
		tiempoRestante = (TextView)findViewById(R.id.textTiempoRestante);
		miEquipo = (TextView)findViewById(R.id.textMiEquipo);
		clasificacion = (ListView)findViewById(R.id.listClasificacion); 
		/*if(! TabHostPartida.TERMINAR_PARTIDA)
			threadReloj.start();*/
	}
	
	@Override
	protected void onResume(){
		super.onResume();
		progressDialog = ProgressDialog.show(this, "", 
				getString(R.string.progDialogRecibDatos), true);
		linearLayout.setVisibility(View.GONE);
		Thread threadObtenerEstadisticas = new Thread(this);
		threadObtenerEstadisticas.start();
	}
	
    /*@Override
    protected void onDestroy(){
    	finThreadReloj = true;
    	super.onDestroy();
    }*/
    
    /*private Thread threadReloj = new Thread(){
    	@Override
    	public void run(){
    		tiempoRestJuego = PartidaListar.getPartida().getDuracion();
    		while((tiempoRestJuego > 0) || ! finThreadReloj){
    			try {
    				handler.sendEmptyMessage(RELOJ);
    				Thread.sleep(60000);
    				tiempoRestJuego = tiempoRestJuego-1;
    			} catch (InterruptedException e) {
    				// TODO Auto-generated catch block
    				e.printStackTrace();
    			}
    		}
    		if(! finThreadReloj)
    			//Terminar juego
    			handler.sendEmptyMessage(TERMINAR_PARTIDA);
    	}
    };*/
	
    @Override
	public void run() {
		Looper.prepare();
		obtenerEstadisticas();
		Looper.myLooper().quit();
		Looper.loop();
	}

	private Handler handler = new Handler(){
		@Override
    	public void handleMessage(Message msg){
			int valor = msg.what;
			int icono;
			String titulo;
			String msgAlert;
			
			progressDialog.cancel();
    		switch(valor){
			case RESULTADO_ERROR_CONEXION:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloConexionServer);
				showAlert(icono,titulo,msgAlert);
			break;
    		case RESULTADO_ERROR:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloDatosRecibidos);
				showAlert(icono,titulo,msgAlert);
    		break;
       		case RESULTADO_OK:
    			miEquipo.setText(infoEquipo);    		
        		AdaptadorClasificacion adaptador = new AdaptadorClasificacion(Estadisticas.this);
        		clasificacion.setAdapter(adaptador);
        	    float density = getResources().getDisplayMetrics().density;
        	    linearLayout.setVisibility(View.VISIBLE);
        	    Utilidades.setListViewHeightBasedOnChildren(clasificacion, density);
       			if(TabHostPartida.getTERMINAR_PARTIDA()){
       				tiempoRestante.setText(String.format(getString(R.string.tiempoRestante), "0"));
       				icono = R.drawable.ic_dialog_info;
    				titulo = getString(R.string.tituloPartidaFin);
    				msgAlert = getString(R.string.textoPartidaFin);
       				showAlert(icono,titulo,msgAlert);
       			}
        	    break;    			
    		/*case RELOJ:    				
    			partidaCrono.setText(String.format(getString(R.string.tiempoRestante), tiempoRestJuego));
    			break;
    		case TERMINAR_PARTIDA:
    			TabHostPartida.TERMINAR_PARTIDA = true;
    			Context context = getApplicationContext();
    			Intent intent = new Intent(context, Estadisticas.class);				
				break;*/
    		};
    	}
	};
	
	private void obtenerEstadisticas(){
		int resultado;
	
		try{
			String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
			int idPartida = PartidaListar.getPartida().getIdent();
			int idEquipo = RecursoListar.getEquipo().getIdent();
			String url = String.format("%1$sconquistadores/partida/%2$s/estadisticas/%3$s/?format=json",
					urlServer,idPartida,idEquipo);
			String str = Utilidades.obtenerDatos(url);
			resultado = procesarRespuestaEstadisticas(str);
			handler.sendEmptyMessage(resultado);
	
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }catch (IOException e) {
		    Log.e(TAG, e.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }
	}
    
    private int procesarRespuestaEstadisticas (String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;
			
			listNombre = new ArrayList<String>();
			listColor = new ArrayList<String>();
			listNumTerritorios = new ArrayList<String>();
			listPuntuacion = new ArrayList<String>();			
			String nombreTerritorios = new String();
			String equipoNombTerritorio;
			
			if (json.has("marcadores")){
				int equipoId;
				int equipoPuntuacion;
				int equipoNumTerritorio;
				
				JSONArray marcadoresObject = json.getJSONArray("marcadores");
				int longitud = marcadoresObject.length()-1;
					
				for (int i = 0; i < longitud; i++){
					JSONObject JSONMarcador = (JSONObject)marcadoresObject.get(i);
					
					equipoId = JSONMarcador.getInt("equipoId");
					equipoNumTerritorio = JSONMarcador.getInt("num_territorios");
					equipoPuntuacion = JSONMarcador.getInt("puntuacion");
					
					Equipo equipo = Equipo.selecionarPorId(PartidaListar.getPartida().getEquipos(), equipoId);
					if (equipo == null)
						return RESULTADO_ERROR;
					
					String equipoNombre = equipo.getGroupName();
					String equipoColor = equipo.getColor();
					
					if ((JSONMarcador.has("territorios")) && (RecursoListar.getEquipo().getIdent() == equipoId)){
						JSONArray territoriosObject = JSONMarcador.getJSONArray("territorios");
						int longitudTerritorios = territoriosObject.length()-1;
						if(longitudTerritorios > 0){
							equipoNombTerritorio = null;
							for (int j = 0; j < longitudTerritorios; j++){
								JSONObject JSONTerritorios = (JSONObject)territoriosObject.get(j);
								equipoNombTerritorio = JSONTerritorios.getString("nombre");
								nombreTerritorios = String.format("%1$s\t%2$s\n",nombreTerritorios,equipoNombTerritorio);		
							}	
						}else{
							nombreTerritorios = ("\t-\n");
						} 
						
						infoEquipo = String.format(getString(R.string.miEquipo),equipoNombre,
								equipoColor,equipoPuntuacion,nombreTerritorios);
						RecursoListar.getEquipo().getMarcador().setNumTerritorios(equipoNumTerritorio);
						RecursoListar.getEquipo().getMarcador().setPuntuacion(equipoPuntuacion);
					}			
							
					listNombre.add(equipoNombre);
					listColor.add(equipoColor);
					listNumTerritorios.add(String.valueOf(equipoNumTerritorio));
					listPuntuacion.add(String.valueOf(equipoPuntuacion));
				}
			}else
				return RESULTADO_ERROR;

		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
        return RESULTADO_OK;
	}
    
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaEstadistica);
        	showAlert(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
	
    @Override
    public boolean onKeyDown (int keyCode, KeyEvent event) {
    	switch(keyCode){
    	case KeyEvent.KEYCODE_BACK:
    	   	AlertDialog.Builder builder = new AlertDialog.Builder(this);
        	builder.setMessage(R.string.abandonarPartida).setCancelable(false)
        		.setPositiveButton(R.string.botonAceptar, new DialogInterface.OnClickListener() {
        			public void onClick(DialogInterface dialog, int id) {
        				Utilidades.finalizarServicios(Estadisticas.this);
        				Utilidades.finalizarActivities((Activity)Estadisticas.this);
        			}
        		})
        		.setNegativeButton(R.string.botonCancelar, new DialogInterface.OnClickListener() {
        			public void onClick(DialogInterface dialog, int id) {
        				dialog.cancel();
        			}
        		});
        		
        	AlertDialog alert = builder.create();
        	alert.setTitle(R.string.abandonar);
        	alert.setIcon(R.drawable.ic_dialog_warning);
        	alert.show();
    		return true;
    	default:
    		return false;
    	}
    }
    
	private void showAlert(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    }
	
    class AdaptadorClasificacion extends BaseAdapter {
    	
    	Context context;
    	
    	AdaptadorClasificacion(Context context) {
    		this.context = context;
    	}
    	
		public int getCount() {
			// TODO Auto-generated method stub
			return listNombre.size();
		}
    	
		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

    	public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) context
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.partida_clasificacion, null);
	        } else {
	            view = convertView;
	        } 
	        
			TextView posicion = (TextView)view.findViewById(R.id.textPosicion);
			posicion.setText(String.valueOf(position+1));
			
			TextView nombre = (TextView)view.findViewById(R.id.textNombre);
			nombre.setText((String)listNombre.get(position));
			
			TextView color = (TextView)view.findViewById(R.id.textColor);			
			if(listColor.get(position).compareTo("amarillo") == 0)
				color.setBackgroundResource(R.color.amarillo);				
			else if(listColor.get(position).compareTo("azul") == 0)
				color.setBackgroundResource(R.color.azul);
			else if(listColor.get(position).compareTo("morado") == 0)
				color.setBackgroundResource(R.color.morado);
			else if(listColor.get(position).compareTo("naranja") == 0)
				color.setBackgroundResource(R.color.naranja);
			else if(listColor.get(position).compareTo("rojo") == 0)
				color.setBackgroundResource(R.color.rojo);
			else if(listColor.get(position).compareTo("verde") == 0)
				color.setBackgroundResource(R.color.verde);
			
			TextView numTerritorios = (TextView)view.findViewById(R.id.textZonas);
			numTerritorios.setText((String)listNumTerritorios.get(position));
					
			TextView puntuacion = (TextView)view.findViewById(R.id.textPuntuacion);
			puntuacion.setText((String)listPuntuacion.get(position));
			
			return view;
		}
    }
}
