package com.Conquistadores.App;

import java.util.Iterator;
import java.util.List;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageButton;

import com.Conquistadores.App.R;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;

/**
 * MapActivity que muestra al usuario la localizacin de los diferentes elementos que
 * conforman la partida: su equipo, los equipos rivales y los territorios.
 * Con pulsar el botn situado abajo a la izquierda, el mapa har focus en la posicin
 * del equipo.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class Mapa extends MapActivity implements OnClickListener{
	
	private final int MENU_AYUDA = Menu.FIRST;
	private static final int ZOOM_LEVEL = 18;
	public static final int RADIO_EQUIPO = 75;
	
	private MapController mc;	
	private List<Overlay> mapOverlays;
	private ImageButton imageButton;	
	private MapView mapView;
	private Equipo miEquipo;
	private MiEquipoItemizedOverlay miEquipoitemizedOverlay;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {	
		super.onCreate(savedInstanceState);
		setContentView(R.layout.partida_mapa);

		imageButton = (ImageButton)this.findViewById(R.id.imageButtonMap);
		imageButton.setOnClickListener(this);
		miEquipo = RecursoListar.getEquipo();
		iniciarMapa();
		iniciarMiEquipo();		
	}
	
	@Override
	protected void onResume() {
		super.onResume();	
		this.registerReceiver(receiver, new IntentFilter("ServicioRecursoMapa.ACCION_ACTUALIZAR_RECURSOS"));
		Intent intent = new Intent(Mapa.this, ServicioMapa.class);
		this.startService(intent);
		centrarEnMiEquipo();
	}
		 
	@Override
	protected void onPause() {
		super.onPause();		
		this.unregisterReceiver(receiver);
		Intent intent = new Intent(Mapa.this, ServicioMapa.class);
		this.stopService(intent);
	}
	
	@Override
	protected boolean isRouteDisplayed() {
	    return false;
	}
	
	private BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String accion = intent.getAction();
            if (accion.compareTo("ServicioRecursoMapa.ACCION_ACTUALIZAR_RECURSOS") == 0){
            	mapOverlays.clear();
        		Location localizacionMiEquipo = ServicioLocalizacion.getLocation();
        		if (localizacionMiEquipo != null){
                	actualizarMiEquipo(localizacionMiEquipo);
                	actualizarTerritorios(localizacionMiEquipo);
        			actualizarEquipos(localizacionMiEquipo);
        		}
            	mapView.invalidate();
            }
        }
    };
	
	private void iniciarMapa(){
		mapView = (MapView) findViewById(R.id.mapView);
		mapView.setSatellite(true);
		mapView.setBuiltInZoomControls(true);
		mapOverlays = mapView.getOverlays();
		mc = mapView.getController();
		mapView.invalidate();
	}
	
	private void centrarEnMiEquipo(){
		Location miLocalizacion = ServicioLocalizacion.getLocation();
		if (miLocalizacion != null){
			GeoPoint miPunto = new GeoPoint((int)(miLocalizacion.getLatitude() * 1E6), (int)(miLocalizacion.getLongitude() * 1E6));
			mc.animateTo(miPunto);
			mc.setZoom(ZOOM_LEVEL);
		}
	}
	
	private void iniciarMiEquipo(){
		Drawable miEquipodrawable = this.getResources().getDrawable(R.drawable.ic_maps_indicator);
		miEquipoitemizedOverlay = new MiEquipoItemizedOverlay(this, miEquipodrawable);
		miEquipoitemizedOverlay.setMeters(Mapa.RADIO_EQUIPO);
	}
	
	private void actualizarMiEquipo(Location localizacionMiEquipo) {		
		GeoPoint punto = new GeoPoint((int)(localizacionMiEquipo.getLatitude() * 1E6), (int)(localizacionMiEquipo.getLongitude() * 1E6));
		OverlayItem overlayitem = new OverlayItem(punto, "", "");
		/*Drawable drawable = this.getResources().getDrawable(R.drawable.ic_maps_indicator);
		MiEquipoItemizedOverlay itemizedOverlay = new MiEquipoItemizedOverlay(this, drawable);
		itemizedOverlay.setMeters(Mapa.RADIO_EQUIPO);
		itemizedOverlay.addOverlay(overlayitem);
		mapOverlays.add(itemizedOverlay);*/
		miEquipoitemizedOverlay.addOverlay(overlayitem);
		mapOverlays.add(miEquipoitemizedOverlay);
	}
	
	private void actualizarTerritorios(Location localizacionMiEquipo){
		Territorio territorio;
		GeoPoint punto;
		OverlayItem overlayitem;
		String color;
		Drawable drawable;
		TerritorioItemizedOverlay itemizedOverlay;	
		
		List<Territorio> territorios = PartidaListar.getPartida().getTerritorios();		
		Iterator<Territorio> it = territorios.iterator();
		while (it.hasNext()){
			territorio = it.next();	
			punto = new GeoPoint((int)(territorio.getLatitude() * 1E6), (int)(territorio.getLongitude() * 1E6));
			overlayitem = new OverlayItem(punto, "", String.valueOf(territorio.getId()));
			color = territorio.getColor();			
			drawable = drawableTerritorio(color);
			itemizedOverlay = new TerritorioItemizedOverlay(this, localizacionMiEquipo, drawable);				
			itemizedOverlay.addOverlay(overlayitem);
			mapOverlays.add(itemizedOverlay);
		}
	}
	
	private Drawable drawableTerritorio(String color){
		Drawable drawable;

		if (color.compareTo("amarillo") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_base_amarillo);
		else if (color.compareTo("azul") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_base_azul);
		else if (color.compareTo("morado") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_base_morado);
		else if (color.compareTo("naranja") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_base_naranja);
		else if (color.compareTo("rojo") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_base_rojo);
		else if (color.compareTo("verde") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_base_verde);
		else
			drawable = this.getResources().getDrawable(R.drawable.ic_base_gris);
		return drawable;
	}
	
	private void actualizarEquipos(Location localizacionMiEquipo){
		Equipo equipo;
		float[] resultados = new float[3];
		float distancia;		
		GeoPoint punto;
		OverlayItem overlayitem;
		String color;
		Drawable drawable;
		EquipoItemizedOverlay itemizedOverlay;		

		List<Equipo> equipos = PartidaListar.getPartida().getEquipos();	
		Iterator<Equipo> it = equipos.iterator();		
		while (it.hasNext()){
			equipo = it.next();
			Location.distanceBetween(localizacionMiEquipo.getLatitude(), localizacionMiEquipo.getLongitude(), 
					equipo.getLatitude(), equipo.getLongitude(), resultados);
			distancia = resultados[0];
			//Localizo los equipos rivales (el mio no) y compruebo que estan dentro del radio de accion de mi equipo.
			if ((equipo.getIdent() != miEquipo.getIdent()) && (distancia <= RADIO_EQUIPO)){
				punto = new GeoPoint((int)(equipo.getLatitude() * 1E6), (int)(equipo.getLongitude() * 1E6));
				overlayitem = new OverlayItem(punto, "", String.valueOf(equipo.getIdent()));
				color = equipo.getColor();			
				drawable = drawableEquipo(color);
				itemizedOverlay = new EquipoItemizedOverlay(this, drawable);				
				itemizedOverlay.addOverlay(overlayitem);
				mapOverlays.add(itemizedOverlay);
			}
		}
	}
	
	private Drawable drawableEquipo(String color){
		Drawable drawable;
		
		if (color.compareTo("amarillo") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_equipo_amarillo);
		else if (color.compareTo("azul") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_equipo_azul);
		else if (color.compareTo("morado") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_equipo_morado);
		else if (color.compareTo("naranja") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_equipo_naranja);
		else if (color.compareTo("rojo") == 0)
			drawable = this.getResources().getDrawable(R.drawable.ic_equipo_rojo);
		else
			drawable = this.getResources().getDrawable(R.drawable.ic_equipo_verde);
		return drawable;
	}
	
	@Override
	public void onClick(View arg0) {
		if(imageButton.isPressed())
			centrarEnMiEquipo();
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaMapa);
        	showAlert(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
	
	@Override
    public boolean onKeyDown (int keyCode, KeyEvent event) {
    	switch(keyCode){
    	case KeyEvent.KEYCODE_BACK:
    	   	AlertDialog.Builder builder = new AlertDialog.Builder(this);
        	builder.setMessage(R.string.abandonarPartida).setCancelable(false)
        		.setPositiveButton(R.string.botonAceptar, new DialogInterface.OnClickListener() {
        			public void onClick(DialogInterface dialog, int id) {
        				Utilidades.finalizarServicios(Mapa.this);
        				Utilidades.finalizarActivities((Activity)Mapa.this);
        			}
        		})
        		.setNegativeButton(R.string.botonCancelar, new DialogInterface.OnClickListener() {
        			public void onClick(DialogInterface dialog, int id) {
        				dialog.cancel();
        			}
        		});
        		
        	AlertDialog alert = builder.create();
        	alert.setTitle(R.string.abandonar);
        	alert.setIcon(R.drawable.ic_dialog_warning);
        	alert.show();
    		return true;
    	default:
    		return false;
    	}
    }
	
	private void showAlert(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    }
}
