
package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.Conquistadores.App.R;

/**
 * Clase que lista, segn la opcin elegida por el usuario, los mensajes recibidos o enviados
 * por el equipo.
 * Si se desea conocer ms detalles de un mensaje, basta con pulsar sobre el mensaje en
 * cuestin y se lanza la actividad MensajeDetalle.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class MensajeListar extends ListActivity implements Runnable{

	static private String TAG = "MensajeListar";
	
	private final int MENU_AYUDA = Menu.FIRST;
	private final int RESULTADO_ERROR_CONEXION = -4;
	private final int RESULTADO_ERROR = -3;
	private final int RESULTADO_NO_MENSAJES = -2;
	private final int RESULTADO_OK = 1;
	
	private static String tipo_mensaje;
	private List<Mensaje> mensajes;
	private static Mensaje[] mensajeRecibidos=null;
	private static Mensaje[] mensajeEnviados=null;
	private static Mensaje mensaje;
	
	static private ArrayList<String> listEmisor;
	static private ArrayList<String> listReceptor;
	static private ArrayList<String> listTexto;
	
	private ProgressDialog progressDialog;
		
	public static Mensaje[] getMensajeRecibidos(){
		return mensajeRecibidos;
	}
	public static Mensaje getMensaje(){
		return mensaje;
	}
	private static String getTipoMensaje(){
		return tipo_mensaje;
	}
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		Bundle extras = getIntent().getExtras();
		MensajeListar.tipo_mensaje = extras.getString("tipo_mensaje");		
		progressDialog = ProgressDialog.show(this, "", 
				getString(R.string.progDialogRecibMensaje), true);
		Thread thread = new Thread(this);
		thread.start();
	}
	
	@Override
	protected void onPause(){
		super.onPause();
		finish();
	}
	
	@Override
	public void run() {
		Looper.prepare();
		obtenerMensajes();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
    private Handler handler = new Handler(){
    	public void handleMessage(Message msg){
			int valor = msg.what;
			int icono;
			String titulo;
			String msgAlert;
			
			progressDialog.cancel();
			switch(valor){
			case RESULTADO_ERROR_CONEXION:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloConexionServer);
				showAlert(icono,titulo,msgAlert);
				break;
			case RESULTADO_ERROR:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloDatosRecibidos);
				showAlert(icono,titulo,msgAlert);
				break;
			case RESULTADO_NO_MENSAJES:
				icono = R.drawable.ic_dialog_info;
				titulo = getString(R.string.info);
				msgAlert = getString(R.string.noMensajes);
				showAlert(icono,titulo,msgAlert);
				break;
    		case RESULTADO_OK:
    			int ultMensajeId = Mensaje.obtnerIdUltMensaje(MensajeListar.getMensajeRecibidos());
    			ServicioMensajeria.setMensajeAplicacionId(ultMensajeId);
    			EventsAdapter adapter = new EventsAdapter(MensajeListar.this);
				setListAdapter(adapter);
				break;
    		};
    	}
    };
	
	private void obtenerMensajes(){
		int  resultado;
	
		try{
			String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
			int partidaId = PartidaListar.getPartida().getIdent();
			int equipoId = RecursoListar.getEquipo().getIdent();
			String tipoMensaje = MensajeListar.getTipoMensaje();
			String url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/mensaje/%4$s/listar/?format=json",
					urlServer,partidaId,equipoId,tipoMensaje);
			String str = Utilidades.obtenerDatos(url);
			resultado = procesarRespuestaMensajeRecibidos(str);
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }catch (IOException e) {
		    Log.e(TAG, e.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }
	}
	
    private int procesarRespuestaMensajeRecibidos (String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;
			
			listEmisor = new ArrayList<String>();
			listReceptor = new ArrayList<String>();
			listTexto = new ArrayList<String>();		
			mensajes = new ArrayList<Mensaje>();
			
			if (json.has("mensajes")){
				int mensajeId;		
				String mensajeEmisor;
				String mensajeReceptor;
				String mensajeTexto;
				String mensajeFecha;
				
				JSONArray mensajesObject = json.getJSONArray("mensajes");
				int longitudMensajes = mensajesObject.length()-1;
				
				if(MensajeListar.getTipoMensaje().compareTo("r") == 0)
					mensajeRecibidos = new Mensaje[longitudMensajes];
				else
					mensajeEnviados = new Mensaje[longitudMensajes];
					
				for (int i = 0; i < longitudMensajes; i++){
					JSONObject JSONMensajeRecibido = (JSONObject)mensajesObject.get(i);
					
					mensajeId = JSONMensajeRecibido.getInt("ident");					
					mensajeEmisor = JSONMensajeRecibido.getString("emisor");
					mensajeReceptor = JSONMensajeRecibido.getString("receptor");
					mensajeTexto = JSONMensajeRecibido.getString("texto");
					mensajeFecha = JSONMensajeRecibido.getString("fecha");

					if (MensajeListar.getTipoMensaje().compareTo("r") == 0){
						mensajeRecibidos[i] = new Mensaje(mensajeId, mensajeEmisor, mensajeReceptor,
							mensajeTexto, mensajeFecha);
						mensajes.add(mensajeRecibidos[i]);
					}else{
						mensajeEnviados[i] = new Mensaje(mensajeId, mensajeEmisor, mensajeReceptor,
							mensajeTexto, mensajeFecha);
						mensajes.add(mensajeEnviados[i]);
					}
					
					listEmisor.add(String.format(getString(R.string.equipoEmisor), mensajeEmisor));
					listReceptor.add(String.format(getString(R.string.equipoReceptor), mensajeReceptor));
					listTexto.add(mensajeTexto);					
					
				}
			}else
				return RESULTADO_NO_MENSAJES;

		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
		return RESULTADO_OK;
	}
    
	private void showAlert(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    }  
    
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaMensajeListar);
    		showAlertAyuda(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }

	private void showAlertAyuda(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    }    
    
	protected void onListItemClick(ListView l, View v, int position, long id) {
		mensaje = mensajes.get(position);
		Intent i = new Intent(MensajeListar.this, MensajeDetalle.class);
		this.startActivity(i);
		MensajeListar.this.setResult(RESULT_OK);
		MensajeListar.this.finish();
	}

    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return listReceptor.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.mensaje_listar, null);
	        } else {
	            view = convertView;
	        }
	        
	        LinearLayout fondo = (LinearLayout) view.findViewById(R.id.LinearLayout1);
	        if(MensajeListar.getTipoMensaje().compareTo("r") == 0)
	        	fondo.setBackgroundDrawable(view.getResources().getDrawable(R.drawable.fondo_verde));
	        else
	        	fondo.setBackgroundDrawable(view.getResources().getDrawable(R.drawable.fondo_rojo));
			
			TextView emisor = (TextView) view.findViewById(R.id.textEmisor);
			emisor.setText((String)listEmisor.get(position));
			
			TextView receptor = (TextView) view.findViewById(R.id.textReceptor);
			receptor.setText((String)listReceptor.get(position));
			
			TextView texto = (TextView) view.findViewById(R.id.textMensaje);
			texto.setText((String)listTexto.get(position));
			
			return view;
		}
	}
}
