package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONObject;

import com.Conquistadores.App.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Activity que permite redactar un nuevo mensaje y elegir el destinatario, ya sea a un
 * equipo o al organizador de la partida, a travs del spinner.
 * Si en el momento de enviarlo, el cuerpo del mensaje est vaco, se muestra un toast
 * al usuario advirtindole del problema.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class MensajeNuevo extends Activity implements Runnable, OnClickListener{

	static private String TAG = "MensajeNuevo";
	
	private final int MENU_AYUDA = Menu.FIRST;
	private final String organizador = "Organizador";
	private final int RESULTADO_ERROR_CONEXION = -3;
	private final int RESULTADO_ERROR = -2;
	private final int RESULTADO_OK = 1;

	private TextView equipoEmisor;
	private TextView equipoReceptor;
	private EditText Mensaje;
	private Button botonEnviar;
	
	private	int idDestino;
	
	private ProgressDialog progressDialog;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.mensaje_nuevo);
		
		equipoEmisor = (TextView) this.findViewById(R.id.textEmisor);
		String emisor = String.format(getString(R.string.equipoEmisor), RecursoListar.getEquipo().getGroupName());
		equipoEmisor.setText(emisor);
		
		equipoReceptor = (TextView) this.findViewById(R.id.textReceptor);
		String receptor = String.format(getString(R.string.equipoReceptor), "");
		equipoReceptor.setText(receptor);
		
		List<String> equipoNombres = new ArrayList<String>();
		equipoNombres.add(organizador);		
		for(int i=0; i<PartidaListar.getPartida().getEquipos().size(); i++){
			equipoNombres.add(i+1, PartidaListar.getPartida().getEquipos().get(i).getGroupName());
		}
	    Spinner s = (Spinner) findViewById(R.id.spinnerReceptor);
	    ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, 
	    		android.R.layout.simple_spinner_item, equipoNombres);
	    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
	    s.setAdapter(adapter);

	    MyOnItemSelectedListener listener = new MyOnItemSelectedListener();
	    s.setOnItemSelectedListener(listener);
	    
	    Mensaje = (EditText) this.findViewById(R.id.editMensaje);	    
	    getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);
	    
	    botonEnviar = (Button)this.findViewById(R.id.botonEnviar);		
	    botonEnviar.setOnClickListener(this);
	}
	
	@Override
	protected void onPause(){
		super.onPause();
		finish();
	}
	
	@Override
	public void run() {
		Looper.prepare();
		enviarMensaje();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	private Handler handler = new Handler(){
		@Override
		public void handleMessage(Message msg){
			int valor = msg.what;
			int icono;
			String titulo;
			String msgAlert;
			
			progressDialog.cancel();
			switch(valor){
			case RESULTADO_ERROR_CONEXION:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloConexionServer);
				showAlert(icono,titulo,msgAlert);
			break;			
			case RESULTADO_ERROR:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloEnvioMensaje);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_OK:
				Context context = getApplicationContext();
				CharSequence text = getString(R.string.toastMensajeEnviado);
				Toast toast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
				MensajeNuevo.this.setResult(RESULT_OK);
				MensajeNuevo.this.finish();
			break;
			};
		}
	};

	private void enviarMensaje(){
		int resultado;
		
		try{
			String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
			int idPartida = PartidaListar.getPartida().getIdent();
			String url = String.format("%1$sconquistadores/partida/%2$s/mensaje/crear/?format=json",
					urlServer, idPartida);
						
			List<NameValuePair> datos = new ArrayList<NameValuePair>();

			BasicNameValuePair bnvp = new BasicNameValuePair("emisor", Integer.toString(RecursoListar.getEquipo().getIdent()));
			datos.add(bnvp);

			//El destinatario no es el organizador de la partida
			if(idDestino >= 0){
				bnvp = new BasicNameValuePair("tipo_receptor", "unico");
				datos.add(bnvp);
				bnvp = new BasicNameValuePair("receptor", Integer.toString(idDestino));
				datos.add(bnvp);
			}else{
				bnvp = new BasicNameValuePair("tipo_receptor", "organizador");
				datos.add(bnvp);
			}
			
			bnvp = new BasicNameValuePair("texto", Mensaje.getText().toString().trim());
			datos.add(bnvp);
			String str = Utilidades.enviarDatos(url, datos);
			resultado = procesarRespuestaMensajeNuevo(str);
			handler.sendEmptyMessage(resultado);	
		
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }catch (IOException e) {
		    Log.e(TAG, e.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }
	}

    private int procesarRespuestaMensajeNuevo (String str) {
    	int code;
    	String info;
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;

		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
		return RESULTADO_OK;
	}
    
	@Override
	public void onClick(View v) {
		if(botonEnviar.isPressed()){ 
			if(Mensaje.getText().toString().trim().equals("")){
				Context context = getApplicationContext();
				CharSequence text = getString(R.string.toastMensajeVacio);
				Toast toast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
			}else{
				progressDialog = ProgressDialog.show(this, "", 
						getString(R.string.progDialogEnviarMensaje), true);
				Thread thread = new Thread(this);
				thread.start();
			}
		}
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaMensajeNuevo);
        	showAlert(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
	private void showAlert(int icono, String titulo, String msg){ 	
		AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
    }
    	
	public class MyOnItemSelectedListener implements OnItemSelectedListener {
		public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
			String destino = parent.getItemAtPosition(pos).toString();
			if(destino.compareTo(organizador) != 0)
				idDestino = Equipo.selecionarPorNombre(PartidaListar.getPartida().getEquipos(), destino).getIdent();
			else
				idDestino = -1;
		}
		
		public void onNothingSelected(AdapterView<?> parent) {
		}
	}
}
