package com.Conquistadores.App;

import com.Conquistadores.App.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageView;

/**
 * Pantalla que permite crear un nuevo mensaje y listar los mensajes recibidos y enviados.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class Mensajeria extends Activity implements OnClickListener{
	//static private String TAG = "Mensajeria";
	
	private final int MENU_AYUDA = Menu.FIRST;
	
	private ImageView mensajeNuevo;
	private ImageView mensajeEnviado;
	private ImageView mensajeRecibido;
	
    @Override
    public void onCreate(Bundle savedInstanceState){
        super.onCreate(savedInstanceState);
        setContentView(R.layout.mensajeria);
        
        mensajeNuevo = (ImageView) this.findViewById(R.id.iconoMensajeNuevo);	
        mensajeNuevo.setOnClickListener(this);
        
        mensajeRecibido = (ImageView) this.findViewById(R.id.iconoMensajeRecibido);	
        mensajeRecibido.setOnClickListener(this);
        
        mensajeEnviado = (ImageView) this.findViewById(R.id.iconoMensajeEnviado);	
        mensajeEnviado.setOnClickListener(this);
    }
       
    public void onClick(View v){ 
    	Intent intent;
    	Bundle bundle = new Bundle();
    	if(mensajeNuevo.isPressed()){ 
    		intent = new Intent(Mensajeria.this, MensajeNuevo.class);
    		this.startActivity(intent);
    		return;
    	}
    	if(mensajeRecibido.isPressed()){ 
    		intent = new Intent(Mensajeria.this, MensajeListar.class);
    		bundle.putString("tipo_mensaje", "r");
    		intent.putExtras(bundle);
    		this.startActivity(intent);
    		return;
    	}
    	if(mensajeEnviado.isPressed()){ 
    		intent = new Intent(Mensajeria.this, MensajeListar.class);
    		bundle.putString("tipo_mensaje", "e");
    		intent.putExtras(bundle);
    		this.startActivity(intent);
    		return;   	
    	}
    }
    
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaMensajeria);
        	showAlert(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    public boolean onKeyDown (int keyCode, KeyEvent event) {
    	switch(keyCode){
    	case KeyEvent.KEYCODE_BACK:
    	   	AlertDialog.Builder builder = new AlertDialog.Builder(this);
        	builder.setMessage(R.string.abandonarPartida).setCancelable(false)
        		.setPositiveButton(R.string.botonAceptar, new DialogInterface.OnClickListener() {
        			public void onClick(DialogInterface dialog, int id) {
        				Utilidades.finalizarServicios(Mensajeria.this);
        				Utilidades.finalizarActivities((Activity)Mensajeria.this);
        			}
        		})
        		.setNegativeButton(R.string.botonCancelar, new DialogInterface.OnClickListener() {
        			public void onClick(DialogInterface dialog, int id) {
        				dialog.cancel();
        			}
        		});
        		
        	AlertDialog alert = builder.create();
        	alert.setTitle(R.string.abandonar);
        	alert.setIcon(R.drawable.ic_dialog_warning);
        	alert.show();
    		return true;
    	default:
    		return false;
    	}
    }
    
    private void showAlert(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    }
}
