package com.Conquistadores.App;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.widget.ListAdapter;

import com.Conquistadores.App.R;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapView;
import com.google.android.maps.OverlayItem;

/**
 * Clase auxiliar para aadir la capa del equipo al MapActivity de la aplicacin.
 * Mediante el mtodo draw, se representa el equipo con un icono personalizado 
 * junto con su radio de accin.
 * Si el usuario pulsa sobre su equipo, mediante el mtodo onTap, se muestra el
 * correspondiente context menu.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class MiEquipoItemizedOverlay extends ItemizedOverlay<OverlayItem>{
		
	private OverlayItem overlayItem;
	private Context context;
	private int meters;

	public MiEquipoItemizedOverlay(Context context, Drawable defaultMarker) {
		super(boundCenter(defaultMarker));
		this.context = context;
	}
		
	@Override
	protected OverlayItem createItem(int i) {
	  return overlayItem;
	}
	
    @Override 
    public void draw(Canvas canvas, MapView mapView, boolean shadow) {
    	GeoPoint geoPoint = overlayItem.getPoint();
        Point screenPoint = mapView.getProjection().toPixels(geoPoint, null);
        
        // Dibujo el circulo azul
        Paint circuloAccion = new Paint();
    	circuloAccion.setAntiAlias(true);
    	circuloAccion.setStrokeWidth(1.0f);
    	circuloAccion.setStyle(Style.FILL_AND_STROKE);
    	circuloAccion.setColor(0xff6666ff);
    	circuloAccion.setAlpha(40);
        
    	int radius = metersToRadius(mapView, geoPoint.getLatitudeE6()/1E6);    			 
        canvas.drawCircle(screenPoint.x, screenPoint.y, radius, circuloAccion);
        
        // Dibujo la flecha que rotara para los dispositivos que lo soporten
        Location location = ServicioLocalizacion.getLocation();
        if (location.hasBearing()){
        	Bitmap flecha = BitmapFactory.decodeResource(context.getResources(),
        			R.drawable.ic_maps_arrow);
        	float angulo = location.getBearing();
        	Matrix matrix = new Matrix();        	
        	matrix.postRotate(angulo);
        	Bitmap flechaRotada = Bitmap.createBitmap(
        			flecha, 0, 0,
        			flecha.getWidth(),
        			flecha.getHeight(),
        			matrix, true);
        	canvas.drawBitmap(flechaRotada, 
        			screenPoint.x - (flechaRotada.getWidth()/2), 
        			screenPoint.y - (flechaRotada.getHeight()/2),
        			null);   	
        }
        super.draw(canvas, mapView, false);
        //mapView.invalidate();
    }
    
    @Override 
    protected boolean onTap(int indice){
    	String titulo = context.getString(R.string.menuEquipo);
    	final MenuFila[] opciones = {new MenuFila(R.drawable.ic_map_mover, context.getString(R.string.menuMovimiento)),
    		new MenuFila(R.drawable.ic_map_equipo_info, context.getString(R.string.menuInfo))};
    	ListAdapter adapter = new MenuAdapter(context, opciones);
    	AlertDialog.Builder builder = new AlertDialog.Builder(context);
    	builder.setAdapter(adapter, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int item) {
            	dialog.dismiss();
            	int equipoId = RecursoListar.getEquipo().getIdent();
            	OpcionesMenuEquipo opcionMenu = new OpcionesMenuEquipo(context, equipoId);
                String opcion = opciones[item].getTitulo();				
	            if(opcion.compareTo(context.getString(R.string.menuMovimiento)) == 0){					
					opcionMenu.mover();	
				}else if(opcion.compareTo(context.getString(R.string.menuInfo)) == 0){
					opcionMenu.informacion();
				}
            }
        });

    	AlertDialog alert = builder.create();
		alert.setTitle(titulo);
    	alert.show(); 
    	return super.onTap(indice);
    }
    
	@Override
	public int size() {
		// TODO Auto-generated method stub
		return 1;
	}

	public void addOverlay(OverlayItem overlay) {
		overlayItem = overlay;
	    populate();
	}
    
    public void setMeters(int meters) {
        this.meters = meters;
    }
    
    public int metersToRadius(MapView mapView, double latitude) {
    	return (int) (mapView.getProjection().metersToEquatorPixels(meters) * (1/ Math.cos(Math.toRadians(latitude))));
    }
 
}
