package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONObject;

import com.Conquistadores.App.R;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

/**
 * Clase que implementa las acciones que tienen asociados los equipos.
 * 	 -Informacin
 *   -Obtener tropas
 *   -Duelo
 * Segn la opcin elegida por el usuario en el contex menu de los equipos, 
 * se llamar al correspondiente mtodo para realizar la accin(consultar al servidor,
 * lanzar un intent, simular un duelo, etc.) y finalmente se muestran los resultados. 
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class OpcionesMenuEquipo {
	static private String TAG = "OpcionesMenuEquipo";
	
	private final int RESULTADO_ERROR_CONEXION = -3;
	private final int RESULTADO_ERROR = -2;
	private final int VERSION_DIFERENTE = -1;
	private final int RESULTADO_OK = 1;
	private final double RANGO = 0.1;
	private final double LIMITE_INFERIOR = 0.05;
	private final double LIMITE_SUPERIOR = 1;
	private final int PUNTOS_DUELO = 15;
	private final int REUTILIZACION_DUELO = 60 * 1000;
	
	private Context context;
	private int equipoDestinoId;
	private int equipoOrigenId;		
	private String datosEquipo;	
	private int equipoDefensorVersion;
	private int equipoAtacanteVersion;
	private int equipoDefensorEscolta; 			
	private int equipoAtacanteEscolta;
	private boolean dueloIgualado;
	private static boolean permitirDuelo = true;
	
	private ProgressDialog progressDialog;
	
	public OpcionesMenuEquipo(Context context, int equipoId){
		this.context=context;
		this.equipoDestinoId=equipoId;
		this.equipoOrigenId = RecursoListar.getEquipo().getIdent();
	}
	
	public void informacion(){
    	progressDialog = ProgressDialog.show(context, "", 
				context.getString(R.string.progDialogRecibDatos), true);
		threadInformacion.start();
    }
    
    private Thread threadInformacion = new Thread(){
    	@Override
    	public void run(){
    		Looper.prepare();
    		obtenerInformacion();
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
    			int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloDatosRecibidos);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_OK:
    				icono = R.drawable.ic_dialog_info;
    				titulo = context.getString(R.string.info);
    				msgAlert = datosEquipo;
    				showAlert(icono,titulo,msgAlert);
    			break;
    			}
    		}
    	};
	
    	private void obtenerInformacion(){
    		int resultado;
    		
    		try{
    			String urlServer = "http://"+context.getString(R.string.urlServer).replaceFirst("/social/", "/");
    			int partidaId = PartidaListar.getPartida().getIdent();
    			String url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/info/?format=json",
    					urlServer, partidaId, equipoDestinoId);		
    			
    			String str = Utilidades.obtenerDatos(url);
    			resultado = procesarRespuestaConquista(str);
    			handler.sendEmptyMessage(resultado);
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaConquista (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200){
    				return RESULTADO_ERROR;
    			}
    			if (json.has("equipo")){
    				String nombre;
    				int escolta;
    				int espias;
    				
    				JSONObject JSONEquipo= json.getJSONObject("equipo");
    				nombre = JSONEquipo.getString("nombre");
    				escolta = JSONEquipo.getInt("escolta");
    				espias = JSONEquipo.getInt("espias");
    					
    				datosEquipo = String.format(context.getString(R.string.equipoInfo), nombre,escolta,espias);
    			}else
    				return RESULTADO_ERROR;
    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
	public void duelo(){
		if (permitirDuelo){
	    	progressDialog = ProgressDialog.show(context, "", 
					context.getString(R.string.progDialogDuelo), true);
			threadDuelo.start();
		}else{
			CharSequence text = context.getString(R.string.noPermisoDuelo);
			Toast toast = Toast.makeText(context, text, Toast.LENGTH_LONG);
			toast.show();
		}
    }
    
    private Thread threadDuelo = new Thread(){
    	String url;
    	
    	@Override    	
    	public void run(){
			String urlServer = "http://"+context.getString(R.string.urlServer).replaceFirst("/social/", "/");
			int partidaId = PartidaListar.getPartida().getIdent();    			
			url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/equipo/%4$s/duelo/?format=json",
					urlServer, partidaId, equipoOrigenId, equipoDestinoId);	
    		Looper.prepare();
    		obtenerDatosDuelo();
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Thread threadPermisoDuelo = new Thread(){
        	@Override
        	public void run(){
        		try {
        			permitirDuelo = false;
					Thread.sleep(REUTILIZACION_DUELO);
					permitirDuelo = true;
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}        		
        	}
        };
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
    			int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.errorDatosDuelo);
    				showAlert(icono,titulo,msgAlert);
    				break;	
    			case VERSION_DIFERENTE:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.errorVersionDuelo);
    				showAlert(icono,titulo,msgAlert);
					break;		
    			case RESULTADO_OK:  
    				CharSequence text;
    				int ganadorId = (Integer)(msg.obj);
    				if(ganadorId == RecursoListar.getEquipo().getIdent())
    					text = context.getString(R.string.exitoDuelo);
    				else
    					if (dueloIgualado)
    						text = context.getString(R.string.fracasoDuelo);
    					else
    						text = context.getString(R.string.fracasoAbsolutoDuelo);
    				Toast toast = Toast.makeText(context, text, Toast.LENGTH_LONG);
    				toast.show();
    				threadPermisoDuelo.start();
    				break;
    			}
    		}
    	};
	
    	private void obtenerDatosDuelo(){
    		int resultado;
    		
    		try{  		
    			String str = Utilidades.obtenerDatos(url);
    			resultado = procesarRespuestaObtenerDatosDuelo(str);
    			
    			if (resultado == RESULTADO_OK)
    				simularDuelo(); 
    			else
    				handler.sendEmptyMessage(resultado);
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaObtenerDatosDuelo (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200)
    				return RESULTADO_ERROR;

    			if (json.has("equipoAtacante") && json.has("equipoDefensor")){    				
    				JSONObject JSONEquipoAtac = json.getJSONObject("equipoAtacante");
    				equipoAtacanteVersion = JSONEquipoAtac.getInt("version");
    				equipoAtacanteEscolta = JSONEquipoAtac.getInt("escolta");
    				
    				JSONObject JSONEquipoDef = json.getJSONObject("equipoDefensor");
    				equipoDefensorVersion = JSONEquipoDef.getInt("version");
    				equipoDefensorEscolta = JSONEquipoDef.getInt("escolta");
    			}else{
    				return RESULTADO_ERROR;
    			}

    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}
        
        private void simularDuelo(){
    		int ganadorId;
    		int defensorEscoltaFinal;
    		int atacanteEscoltaFinal;    	
    		int escoltaTotal = equipoAtacanteEscolta + equipoDefensorEscolta;
    		double porcVictAtacante = (double)(equipoAtacanteEscolta)/escoltaTotal;
    		double porcVictDefensor = 1 - porcVictAtacante;
    		
    		double porcBatalla = Math.random();
    		if(equipoAtacanteEscolta >= (equipoDefensorEscolta*2)){
    			ganadorId =  equipoOrigenId;	
    		}else{
    			if(equipoDefensorEscolta >= (equipoAtacanteEscolta*2))
    				ganadorId =  equipoDestinoId;
    			else{
    				if (porcVictAtacante >= porcBatalla)
    					ganadorId =  equipoOrigenId;
    				else
    					ganadorId =  equipoDestinoId;
    			}
    		}
    		
			dueloIgualado = !((equipoAtacanteEscolta >= (equipoDefensorEscolta*2)) || (equipoDefensorEscolta >= (equipoAtacanteEscolta*2)));
    		double rango = RANGO/2;
    		//Log.e(TAG, "ATACANTE: "+porcVictAtacante+" DEFENSOR: "+porcVictDefensor+" TIRADA: "+porcBatalla);
			double limiteInferior = porcVictDefensor - rango;
			if (limiteInferior < LIMITE_INFERIOR)
				limiteInferior = LIMITE_INFERIOR;
			double limiteSuperior = porcVictDefensor + rango;
			if (limiteSuperior > LIMITE_SUPERIOR)
				limiteSuperior = LIMITE_SUPERIOR;
			double aleatoriedad = Math.random()*(limiteSuperior - limiteInferior)+(limiteInferior);
    		defensorEscoltaFinal = (int)(equipoDefensorEscolta*aleatoriedad);
    		//Log.e(TAG, "ALET_DEFEN: "+aleatoriedad+" ESCOLTA_DEF: "+defensorEscoltaFinal);
			limiteInferior = porcVictAtacante - rango;
			if (limiteInferior < LIMITE_INFERIOR)
				limiteInferior = LIMITE_INFERIOR;
			limiteSuperior = porcVictAtacante + rango;
			if (limiteSuperior > LIMITE_SUPERIOR)
				limiteSuperior = LIMITE_SUPERIOR;
			aleatoriedad = Math.random()*(limiteSuperior - limiteInferior)+(limiteInferior);
    		atacanteEscoltaFinal = (int)(equipoAtacanteEscolta*aleatoriedad);
    		//Log.e(TAG, "ALET_ATAC: "+aleatoriedad+" ESCOLTA_ATAC: "+atacanteEscoltaFinal);
    		EnviarDatosDuelo(ganadorId, defensorEscoltaFinal, atacanteEscoltaFinal);	
    	}
        
        private void EnviarDatosDuelo(int ganadorId, int defensorEscoltaFinal, int atacanteEscoltaFinal){
    		int resultado;
    		
    		try{    		
    			List<NameValuePair> datos = new ArrayList<NameValuePair>();
    			BasicNameValuePair bnvp = new BasicNameValuePair("ganadorId", Integer.toString(ganadorId));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("equipoDefensorVersion", Integer.toString(equipoDefensorVersion));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("equipoAtacanteVersion", Integer.toString(equipoAtacanteVersion));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("atacanteEscoltaFinal", Integer.toString(atacanteEscoltaFinal));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("defensorEscoltaFinal", Integer.toString(defensorEscoltaFinal));
    			datos.add(bnvp);		
    			bnvp = new BasicNameValuePair("puntuacion", Integer.toString(PUNTOS_DUELO));
    			datos.add(bnvp);
    			String str = Utilidades.enviarDatos(url, datos);
    			resultado = procesarRespuestaEnviarDatosDuelo(str);
    			  		
    			Message message = Message.obtain();
    			message.obj = ganadorId;
    			message.what = resultado;
    			handler.sendMessage(message);
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaEnviarDatosDuelo (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200){
    				if (info.toLowerCase().compareTo("version diferente") == 0)
    					return VERSION_DIFERENTE;
    				return RESULTADO_ERROR;
    			}

    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}
    };
    
	public void mover(){
    	Intent intent = new Intent(context, TropaMover.class);
    	Bundle bundle = new Bundle();
    	bundle.putInt("destinoId", equipoDestinoId);
    	bundle.putString("tipo_destino", "equipo");
    	intent.putExtras(bundle);
    	context.startActivity(intent);
	}
	
	private void showAlert(int icono, String titulo, String msg){ 	
		AlertDialog.Builder adb = new AlertDialog.Builder(context);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(context.getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
    }

}
