package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

import com.Conquistadores.App.R;

/**
 * Clase que implementa las acciones que tienen asociados los territorios.
 * 	 -Conquistar
 *   -Obtener tropas
 *   -Espiar
 *   -Informacin
 * Segn la opcin elegida por el usuario en el contex menu de los territorios, 
 * se llamar al correspondiente mtodo para realizar la accin(consultar al servidor,
 * lanzar un intent, etc.) y finalmente se muestran los resultados.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class OpcionesMenuTerritorio {
	static private String TAG = "OpcionesMenuTerritorio";
	
	private final int RESULTADO_ERROR_CONEXION = -5;
	private final int RESULTADO_ERROR = -4;
	private final int NO_ESPIAS = -3;
	private final int VERSION_DIFERENTE = -2;
	private final int RESULTADO_OK = 1;
	public static final int PUNTOS_CONQUISTA = 25;
	
	private Context context;
	private int equipoDefensorId;
	private int territorioId;
	private ProgressDialog progressDialog;
	
	private String datosTerritorio;
	
	public OpcionesMenuTerritorio(Context context, int territorioId, int equipoDefensorId){
		this.context=context;
		this.territorioId=territorioId;
		this.equipoDefensorId=equipoDefensorId;
	}
	
	public OpcionesMenuTerritorio(Context context, int territorioId){
		this.context=context;
		this.territorioId=territorioId;
	}
	
	public void conquistar(){
		if (equipoDefensorId == -1){
			progressDialog = ProgressDialog.show(context, "", 
					context.getString(R.string.progDialogConquistar), true);
			threadConquistar.start();
    	}else{
    		Intent intent = new Intent(context, TerritorioRivalConquistar.class);
    		Bundle bundle = new Bundle();
    		bundle.putString("territorioId", Integer.toString(territorioId));
    		bundle.putInt("equipoDefensorId", equipoDefensorId);
    		intent.putExtras(bundle);
    		context.startActivity(intent);
    	}
	}
	
	private Thread threadConquistar = new Thread(){
    	@Override
    	public void run(){
    		Looper.prepare();
    		enviarConquista();
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
        		int valor = msg.what;    			
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.errorDatosConquista);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case VERSION_DIFERENTE:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.errorVersionConquista);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case RESULTADO_OK:
    				CharSequence text = context.getString(R.string.exitoConquista);
    				Toast toast = Toast.makeText(context, text, Toast.LENGTH_LONG);
    				//toast.setGravity(Gravity.CENTER, 0, 0);
    				toast.show();
    			break;
    			}
    		}
    	};
	
    	private void enviarConquista(){
    		int resultado;
    		
    		try{
    			String urlServer = "http://"+context.getString(R.string.urlServer).replaceFirst("/social/", "/");
    			int partidaId = PartidaListar.getPartida().getIdent();
    			int equipoId = RecursoListar.getEquipo().getIdent();
    			String url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/territorio/%4$s/conquistar/?format=json",
    					urlServer, partidaId, equipoId, territorioId);		
    			List<NameValuePair> datos = new ArrayList<NameValuePair>();
    			BasicNameValuePair bnvp = new BasicNameValuePair("equipoDefensorId", Integer.toString(equipoDefensorId));
    			datos.add(bnvp);	
    			bnvp = new BasicNameValuePair("puntuacion", Integer.toString(PUNTOS_CONQUISTA));
    			datos.add(bnvp);
    			String str = Utilidades.enviarDatos(url, datos);
    			resultado = procesarRespuestaConquista(str);
    			handler.sendEmptyMessage(resultado);
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaConquista (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200){
    				if (info.toLowerCase().compareTo("version diferente") == 0)
    					return VERSION_DIFERENTE;
    				return RESULTADO_ERROR;
    			}
    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
    public void informacion(){
    	progressDialog = ProgressDialog.show(context, "", 
				context.getString(R.string.progDialogRecibDatos), true);
		threadInformacion.start();
    }
    
    private Thread threadInformacion = new Thread(){
    	@Override
    	public void run(){
    		Looper.prepare();
    		obtenerInformacion();
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
        		int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloDatosRecibidos);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case VERSION_DIFERENTE:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.errorVersionTerritorioInfo);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case RESULTADO_OK:
    				icono = R.drawable.ic_dialog_info;
    				titulo = context.getString(R.string.info);
    				msgAlert = datosTerritorio;
    				showAlert(icono,titulo,msgAlert);
    			break;
    			}
    		}
    	};
	
    	private void obtenerInformacion(){
    		int resultado;
    		
    		try{
    			String urlServer = "http://"+context.getString(R.string.urlServer).replaceFirst("/social/", "/");
    			int partidaId = PartidaListar.getPartida().getIdent();
    			int equipoId = RecursoListar.getEquipo().getIdent();
    			String url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/territorio/%4$s/info/?format=json",
    					urlServer, partidaId, equipoId, territorioId);		
    			
    			String str = Utilidades.obtenerDatos(url);
    			resultado = procesarRespuestaConquista(str);
    			handler.sendEmptyMessage(resultado);
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaConquista (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200){
    				if (info.toLowerCase().compareTo("version diferente") == 0)
    					return VERSION_DIFERENTE;
    				return RESULTADO_ERROR;
    			}
    			if (json.has("tropas")){
    				String territorioNombre;
    				String tropaNombre;
    				int tropaCantidad;
    				
    				JSONArray tropasObject = json.getJSONArray("tropas");
    				int longitud = tropasObject.length()-1;    					
    				for (int i = 0; i < longitud; i++){
    					JSONObject JSONTropa = (JSONObject)tropasObject.get(i);
    					
    					territorioNombre = JSONTropa.getString("territorioNombre");
    					tropaNombre = JSONTropa.getString("tipo");
    					tropaCantidad = JSONTropa.getInt("cantidad");
    					
    					datosTerritorio = String.format(context.getString(R.string.territorioInfo), territorioNombre,tropaNombre,tropaCantidad);
    				}
    			}else{
    				return RESULTADO_ERROR;
    			}
    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
    public void espiar(){
    	progressDialog = ProgressDialog.show(context, "", 
				context.getString(R.string.progDialogEspiar), true);
    	threadEspiar.start();
    }
    
    private Thread threadEspiar = new Thread(){
    	int numEspias = RecursoListar.getEquipo().getEspias();
    	@Override
    	public void run(){
    		Looper.prepare();
    		if(numEspias > 0)
    			obtenerInformacion();
    		else
    			handler.sendEmptyMessage(NO_ESPIAS);
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
    			int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.falloDatosRecibidos);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case NO_ESPIAS:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = context.getString(R.string.aviso);
    				msgAlert = context.getString(R.string.noEspias);
    				showAlert(icono,titulo,msgAlert);
    				break;

    			case RESULTADO_OK:
    				RecursoListar.getEquipo().setEspias(numEspias - 1);
    				icono = R.drawable.ic_dialog_info;
    				titulo = context.getString(R.string.info);
    				msgAlert = datosTerritorio;
    				showAlert(icono,titulo,msgAlert);
    			break;
    			}
    		}
    	};
	
    	private void obtenerInformacion(){
    		int resultado;
    		
    		try{
    			String urlServer = "http://"+context.getString(R.string.urlServer).replaceFirst("/social/", "/");
    			int partidaId = PartidaListar.getPartida().getIdent();
    			int equipoId = RecursoListar.getEquipo().getIdent();
    			String url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/territorio/%4$s/espiar/?format=json",
    					urlServer, partidaId, equipoId, territorioId);		
    			
    			String str = Utilidades.obtenerDatos(url);
    			resultado = procesarRespuestaConquista(str);
    			handler.sendEmptyMessage(resultado);
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaConquista (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200){
    				if (info.toLowerCase().compareTo("no espias") == 0)
    					return NO_ESPIAS;
    				return RESULTADO_ERROR;
    			}
    			if (json.has("tropas")){
    				String equipoPoseedor;
    				String territorioNombre;
    				String tropaNombre;
    				int tropaCantidad;
    				
    				JSONArray tropasObject = json.getJSONArray("tropas");
    				int longitud = tropasObject.length()-1;    					
    				for (int i = 0; i < longitud; i++){
    					JSONObject JSONTropa = (JSONObject)tropasObject.get(i);
    					
    					equipoPoseedor = JSONTropa.getString("equipoPoseedor");
    					territorioNombre = JSONTropa.getString("territorioNombre");
    					tropaNombre = JSONTropa.getString("tipo");
    					tropaCantidad = JSONTropa.getInt("cantidad");
    					
    					datosTerritorio = String.format(context.getString(R.string.territorioEspionaje), equipoPoseedor, territorioNombre,tropaNombre,tropaCantidad);
    				}
    			}else
    				return RESULTADO_ERROR;
    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
	public void mover(){
    	Intent intent = new Intent(context, TropaMover.class);
    	Bundle bundle = new Bundle();
		bundle.putInt("destinoId", territorioId);
		bundle.putString("tipo_destino", "territorio");
		intent.putExtras(bundle);
    	context.startActivity(intent);
	}
    
	private void showAlert(int icono, String titulo, String msg){ 	
		AlertDialog.Builder adb = new AlertDialog.Builder(context);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(context.getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
    }
}
