
package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.Conquistadores.App.R;

/**
 * Se muestra todas las partidas preparadas para ser jugadas. En caso de que no haya ninguna,
 * se mostrar un aviso al usuario y se volver a la activity inicial.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class PartidaListar extends ListActivity implements Runnable{

	static private String TAG = "PartidaListar";
	
	private final int RESULTADO_ERROR_CONEXION = -3;
	private final int RESULTADO_ERROR = -2;
	private final int RESULTADO_NO_PARTIDAS = -1;
	private final int RESULTADO_OK = 1;
	
	/* Partidas disponibles para los jugadores */
	private List<Partida> partidas;
	private static Partida partida;

	static private ArrayList<String> listNombre;
	static private ArrayList<String> listLugar;
	static private ArrayList<String> listFecha;
	static private ArrayList<String> listCrecimiento;
	static private ArrayList<String> listDuracion;
	
	private ProgressDialog progressDialog;
	
	public static Partida getPartida(){
		return partida;
	}
		
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		progressDialog = ProgressDialog.show(this, "", 
				getString(R.string.progDialogRecibPartidas), true);
		Thread thread = new Thread(this);
		thread.start();
	}
	
	@Override
	public void run() {
		Looper.prepare();
		obtenerPartidas();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	private Handler handler = new Handler(){
		@Override
		public void handleMessage(Message msg){
			int valor = msg.what;
			int icono;
			String titulo;
			String msgAlert;
			
			progressDialog.cancel();
			switch(valor){
			case RESULTADO_ERROR_CONEXION:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloConexionServer);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_ERROR:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloDatosRecibidos);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_NO_PARTIDAS:
				icono = R.drawable.ic_dialog_info;
				titulo = getString(R.string.info);
				msgAlert = getString(R.string.noPartidas);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_OK:
				EventsAdapter adapter = new EventsAdapter(PartidaListar.this);
				setListAdapter(adapter);
			break;
			};
		}
	};

	private void obtenerPartidas(){
		int resultado;
		try{
			String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");		
			String url = String.format("%1$sconquistadores/partida/listar/?format=json",
					urlServer);
			String str = Utilidades.obtenerDatos(url);
			resultado = procesarRespuestaPartidaListar(str);			
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }catch (IOException e) {
		    Log.e(TAG, e.getMessage());
		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }
	}
		
    private int procesarRespuestaPartidaListar (String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;
			
			listNombre = new ArrayList<String>();
			listLugar = new ArrayList<String>();
			listFecha = new ArrayList<String>();
			listCrecimiento = new ArrayList<String>();
			listDuracion = new ArrayList<String>();
			partidas = new ArrayList<Partida>();
			
			if (json.has("partidas")){
				int partidaId;
				String partidaNombre;
				String partidaLugar;
				String partidaFecha;
				int partidaDuracion;
				int partidaNumEspias;
				int partidaNumEscolta;
				String partidaCrecimiento;
				String partidaDescripcion;
				
				JSONArray partidasObject = json.getJSONArray("partidas");
				int longitudPartidas = partidasObject.length()-1;
				if (longitudPartidas <= 0)
					return RESULTADO_NO_PARTIDAS;
										
				for (int i = 0; i < longitudPartidas; i++){
					JSONObject JSONPartida = (JSONObject)partidasObject.get(i);
					
					partidaId = JSONPartida.getInt("ident");
					partidaNombre = JSONPartida.getString("titulo");
					partidaLugar = JSONPartida.getString("lugar");
					partidaFecha = JSONPartida.getString("fecha");
					partidaNumEspias = JSONPartida.getInt("num_espias");
					partidaNumEscolta = JSONPartida.getInt("num_escolta");
					partidaDuracion = JSONPartida.getInt("duracion");
					partidaCrecimiento = JSONPartida.getString("crecimiento");
					partidaDescripcion = JSONPartida.getString("descripcion");
										
					//Formo el objeto y lo almaceno en un Vector<Partida>
					partidas.add(new Partida(partidaId, partidaNombre,partidaLugar, 
							partidaFecha, partidaNumEspias, partidaNumEscolta,
							partidaDuracion, partidaCrecimiento, partidaDescripcion));
					
					// Doy formato a las variables que se van a mostrar en la IU
					listNombre.add(String.format(getString(R.string.nombrePartida), partidaNombre));
					listLugar.add(String.format(getString(R.string.lugarPartida), partidaLugar));
					listFecha.add(String.format(getString(R.string.fechaPartida), partidaFecha));
					listCrecimiento.add(String.format(getString(R.string.crecimientoPartida), partidaCrecimiento));
					listDuracion.add(String.format(getString(R.string.duracionPartida), partidaDuracion));
				}
			}else
				return RESULTADO_NO_PARTIDAS;

        } catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
		return RESULTADO_OK;
	}
    
	protected void onListItemClick(ListView l, View v, int position, long id) {
		partida = partidas.get(position);		
		Intent intent = new Intent(PartidaListar.this, RecursoListar.class);
		
		this.startActivity(intent);
		PartidaListar.this.setResult(RESULT_OK);
		PartidaListar.this.finish();
	}
    
	private void showAlert(int icono, String titulo, String msg){    	
		AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   PartidaListar.this.setResult(RESULT_OK);
		        	   PartidaListar.this.finish();
		           }
		       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
	}
	    
    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return listNombre.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.partida_listar, null);
	        } else {
	            view = convertView;
	        }    
			
			TextView titulo = (TextView) view.findViewById(R.id.textNombrePartida);
			titulo.setText((String)listNombre.get(position));
			
			TextView lugar = (TextView) view.findViewById(R.id.textLugarPartida);
			lugar.setText((String)listLugar.get(position));
			
			TextView fecha = (TextView) view.findViewById(R.id.textFechaPartida);
			fecha.setText((String)listFecha.get(position));
			
			TextView espias = (TextView) view.findViewById(R.id.textCrecimientoPartida);
			espias.setText((String)listCrecimiento.get(position));
			
			TextView duracion = (TextView) view.findViewById(R.id.textDuracionPartida);
			duracion.setText((String)listDuracion.get(position));
			
			return view;
		}
	}
}
