
package com.Conquistadores.App;

import java.io.IOException;

import java.net.URISyntaxException;
import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.Conquistadores.App.R;

/**
 * Se obtienen los recursos principales de la partida seleccionada(territorios y equipos).
 * Si falta alguno de estos, se avisa al usuario y se retorna a la activity inicial.
 * Si esta todo correcto, se guardan los recursos en las listas destinadas a ello de la
 * clase Partida.
 * Al final, se listan los equipos disponibles para la partida seleccionada y el jugador 
 * selecciona el equipo al que desea unirse.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class RecursoListar extends ListActivity implements Runnable{

	static private String TAG = "RecursoListar";
	
	private final int RESULTADO_ERROR_CONEXION = -4;
	private final int RESULTADO_ERROR = -3;
	private final int RESULTADO_NO_TERRITORIOS = -2;
	private final int RESULTADO_NO_EQUIPOS = -1;
	private final int RESULTADO_OK = 1;
	
	private static Equipo equipo;
	
	static private ArrayList<String> listNombre;
	static private ArrayList<String> listColor;
	static private String partidaPert;
	
	private ProgressDialog progressDialog;
	
	public static Equipo getEquipo(){
		return equipo;
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		progressDialog = ProgressDialog.show(this, "", 
				getString(R.string.progDialogRecibRecursos), true);
		Thread thread = new Thread(this);
		thread.start();
	}
	
	@Override
	public void run() {
		Looper.prepare();
		obtenerRecursos();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	private Handler handler = new Handler(){
		@Override
		public void handleMessage(Message msg){
			int valor = msg.what;
			int icono;
			String titulo;
			String msgAlert;
			progressDialog.cancel();
			switch(valor){
			case RESULTADO_ERROR_CONEXION:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloConexionServer);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_ERROR:
				icono = R.drawable.ic_dialog_warning;
				titulo = getString(R.string.aviso);
				msgAlert = getString(R.string.falloDatosRecibidos);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_NO_TERRITORIOS:
				icono = R.drawable.ic_dialog_info;
				titulo = getString(R.string.info);
				msgAlert = getString(R.string.noTerritoriosPartida);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_NO_EQUIPOS:
				icono = R.drawable.ic_dialog_info;
				titulo = getString(R.string.info);
				msgAlert = getString(R.string.noEquiposPartida);
				showAlert(icono,titulo,msgAlert);
			break;
			case RESULTADO_OK:
				EventsAdapter adapter = new EventsAdapter(RecursoListar.this);
				setListAdapter(adapter);
			break;
			};
		}
	};	

	private void obtenerRecursos(){
		int resultado;
		
		try{
			String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
			int idPartida = PartidaListar.getPartida().getIdent();			
			String url = String.format("%1$sconquistadores/partida/%2$s/recurso/listar/?format=json",
					urlServer,idPartida);
			String str = Utilidades.obtenerDatos(url);
			resultado = procesarRespuestaEquipoListar(str);
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
            Log.e(TAG, urisex.getMessage());
            handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }catch (IOException e) {
            Log.e(TAG, e.getMessage());
            handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
	    }
	}
	    
    private int procesarRespuestaEquipoListar (String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;
			
			listNombre = new ArrayList<String>();
			listColor = new ArrayList<String>();		
			PartidaListar.getPartida().setEquipos(new ArrayList<Equipo>());
			PartidaListar.getPartida().setTerritorios(new ArrayList<Territorio>());
			
			if (json.has("equipos") && json.has("territorios")){
				int id;
				int grupoId;
				int equipoPoseedorId;
				String nombre;
				String color;
				double latitud = 0.0;
				double longitud = 0.0;
				double altitud = 0.0;
				double radio = 0.0;
				String since = null;
				
				JSONArray equiposObject = json.getJSONArray("equipos");
				int longitudEquipos = equiposObject.length()-1;
				
				partidaPert = String.format(getString(R.string.partidaPert), PartidaListar.getPartida().getTitulo());
				
				for (int i = 0; i < longitudEquipos; i++){
					JSONObject JSONEquipo = (JSONObject)equiposObject.get(i);
					
					id = JSONEquipo.getInt("ident");
					grupoId = JSONEquipo.getInt("grupoId");
					nombre = JSONEquipo.getString("nombre");
					color = JSONEquipo.getString("color");
										
					Marcador marcador = new Marcador(0,0);
					int espias = PartidaListar.getPartida().getNumEspias();
					int escolta = PartidaListar.getPartida().getNumEscolta();
					PartidaListar.getPartida().getEquipos().add(new Equipo(grupoId, latitud, 
							longitud, altitud, radio, since, nombre, id, color, espias, 
							escolta, marcador));				
					
					listNombre.add(String.format(getString(R.string.nombreEquipo), nombre));
					listColor.add(String.format(getString(R.string.colorEquipo), color));
				}
				
				JSONArray territoriosObject = json.getJSONArray("territorios");
				int longitudTerritorios = territoriosObject.length()-1;				
				
				for (int i = 0; i < longitudTerritorios; i++){
					JSONObject JSONTerritorio = (JSONObject)territoriosObject.get(i);
					
					id = JSONTerritorio.getInt("ident");
					nombre = JSONTerritorio.getString("nombre");
					latitud = JSONTerritorio.getDouble("latitud");
					longitud = JSONTerritorio.getDouble("longitud");
					equipoPoseedorId = JSONTerritorio.getInt("equipoId");
					if (equipoPoseedorId < 0)
						color = "gris";
					else
						color = JSONTerritorio.getString("color");
					PartidaListar.getPartida().getTerritorios().add(new Territorio(id, latitud, 
							longitud, altitud, radio, since, nombre, equipoPoseedorId, color));				
				}
			}else{
				if (info.toLowerCase().compareTo("no equipos") == 0)
					return RESULTADO_NO_EQUIPOS;
				else
					return RESULTADO_NO_TERRITORIOS;
			}

		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
        return RESULTADO_OK;
	}
    
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		equipo = PartidaListar.getPartida().getEquipos().get(position);
		Intent intent = new Intent(RecursoListar.this, PartidaEmpezar.class);
		this.startActivity(intent);
		RecursoListar.this.setResult(RESULT_OK);
		RecursoListar.this.finish();
	}
		
	private void showAlert(int icono, String titulo, String msg){    	
    	AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
    	       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   RecursoListar.this.setResult(RESULT_OK);
    	        	   RecursoListar.this.finish();
    	           }
    	       });
    	AlertDialog alert = adb.create();
    	alert.setTitle(titulo);
    	alert.setIcon(icono);
    	alert.show();
    } 

    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return listNombre.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.equipo_listar, null);
	        } else {
	            view = convertView;
	        }    
			
			TextView nombre = (TextView) view.findViewById(R.id.textNombreEquipo);
			nombre.setText((String)listNombre.get(position));
			
			TextView color = (TextView) view.findViewById(R.id.textColorEquipo);
			color.setText((String)listColor.get(position));
			
			TextView partida = (TextView) view.findViewById(R.id.textPartidaPert);
			partida.setText(partidaPert);
			
			return view;
		}
	}
}
