package com.Conquistadores.App;

import java.io.IOException;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;

import org.json.JSONObject;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

import com.Conquistadores.App.R;

/**
 * Servicio que se ejecuta peridicamente encargndose de guardar y enviar la localizacin 
 * del equipo al servidor.
 * 
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class ServicioLocalizacion extends Service {
	private static String TAG = "ServicioLocalizacion";
	
	private static final int TIEMPO = 5 * 1000;
	private static final int DISTANCIA = 7;
	
	private static Location location;
	private LocationManager locationManager;
	private MyLocationListener myLocationListener;
	private String url;
		
	public static Location getLocation() {
		return location;
	}
	public static void setLocation(Location location) {
		ServicioLocalizacion.location = location;
	}

	@Override
	public void onCreate(){
		super.onCreate();
		// Iniciamos el servicio
		iniciarServicio();
	}

	@Override
	public void onDestroy(){
		super.onDestroy();
		// Detenemos el servicio
		finalizarServicio();
	}
	
	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}
	
	private void iniciarServicio(){
		locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
		ServicioLocalizacion.location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);		
		myLocationListener = new MyLocationListener();
		locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
				TIEMPO, DISTANCIA, myLocationListener);
		
		/*ServicioLocalizacion.location = locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
		locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
				TIEMPO, DISTANCIA, myLocationListener);*/
		
		String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
		int partidaId = PartidaListar.getPartida().getIdent();
		int equipoId = RecursoListar.getEquipo().getIdent();
		url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/servicio/localizacion/?format=json",
				urlServer, partidaId, equipoId);		
	}
	
	private void finalizarServicio(){
		if (locationManager != null)
			locationManager.removeUpdates(myLocationListener);
	}
	
    private class MyLocationListener implements LocationListener, Runnable {
		@Override
		public void onLocationChanged(Location location) {
			if (location != null){
				ServicioLocalizacion.setLocation(location);			
				Thread thread = new Thread(this);
				thread.start();
			}
		}

		@Override
		public void onProviderDisabled(String provider) {
			// TODO Auto-generated method stub			
		}

		@Override
		public void onProviderEnabled(String provider) {
			// TODO Auto-generated method stub			
		}

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {
			// TODO Auto-generated method stub			
		}

		@Override
		public void run() {
			// TODO Auto-generated method stub
			enviarLocalizacion();
		}
		
		private void enviarLocalizacion(){
			try{		
				List<NameValuePair> datos = new ArrayList<NameValuePair>();				
				BasicNameValuePair bnvp = new BasicNameValuePair("latitud", Double.toString(ServicioLocalizacion.getLocation().getLatitude()));
				datos.add(bnvp);
				bnvp = new BasicNameValuePair("longitud", Double.toString(ServicioLocalizacion.getLocation().getLongitude()));
				datos.add(bnvp);			
	
				String str = Utilidades.enviarDatos(url, datos);
				procesarRespuestaLocalizacion(str);
				
			}catch(URISyntaxException urisex){
	            Log.e(TAG, urisex.getMessage());
		    }catch (IOException e) {
	            Log.e(TAG,e.getMessage());
		    }
		}
	    
	    private boolean procesarRespuestaLocalizacion (String str) {
	    	int code;
	    	String info;
	        try {
	        	JSONObject json;
	        	json = new JSONObject(str.toString());

				if (json.has("codigo"))
					code = json.getInt("codigo");
				else
					code = -1;

				if (json.has("info"))
					info = json.getString("info");
				else
					info = null;
				
				if (code != 200)
					return false;

			} catch (Exception e) {	
				code = 500;
				info = "Error en el parsed json: " + e.getMessage();
				Log.e(TAG, info);
				return false;
			}
			return true;
		}    	
    }

}
