package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

import com.Conquistadores.App.R;

/**
 * Servicio que se ejecuta peridicamente para obtener las ltimas actualizaciones de los
 * equipos y territorios de la partida. Si todo ha ido correcto, mediante un broadcast se 
 * da la orden para posicionar y actualizar los territorios y equipos en el mapa.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class ServicioMapa extends Service{
	static private String TAG = "ServicioMapa";
	
	private static final int RETARDO = 0;
	private static final int PERIODO = 5 * 1000;
	private static final int RESULTADO_ERROR = -1;
	private static final int RESULTADO_OK = 1;

	private String url;
	private Timer timer;
		
	@Override
	public void onCreate(){
		super.onCreate();
		// Iniciamos el servicio
		this.iniciarServicio();
	}
  
	@Override
	public void onDestroy(){
		super.onDestroy();
		// Detenemos el servicio
		this.finalizarServicio();
	}
	
	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}

	private void iniciarServicio(){
		String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");		
		int partidaId = PartidaListar.getPartida().getIdent();
		url = String.format("%1$sconquistadores/partida/%2$s/servicio/mapa/?format=json",
				urlServer,partidaId);
		
		TimerTask timerTask = new TimerTask(){
			public void run(){
				obtenerRecursoMapa();
			}
		};
		
		// Creamos el timer
		timer = new Timer();
		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
	}

	private void finalizarServicio(){
		// Detenemos el timer
		if (timer != null)
			this.timer.cancel();
	}
	
    private Handler handler = new Handler(){
    	public void handleMessage(Message msg){
    		int valor = msg.what;
			Context context = getApplicationContext();
    		switch(valor){
    		case RESULTADO_ERROR:
    			CharSequence text = getString(R.string.toastFalloServMapa);
    			Toast toast = Toast.makeText(context, text,  Toast.LENGTH_SHORT);
    			toast.setGravity(Gravity.CENTER, 0, 0);
    			toast.show();
    			break;
    		case RESULTADO_OK:    			
    			Intent intent = new Intent("ServicioRecursoMapa.ACCION_ACTUALIZAR_RECURSOS");
    			context.sendBroadcast(intent,null);
    			break;
    		};
    	}
    };
	
	private void obtenerRecursoMapa(){
		int resultado;

		try{
			String str = Utilidades.obtenerDatos(url);
			resultado = procesarRespuestaRecurosRecibidos(str);
			handler.sendEmptyMessage(resultado);
			
		}catch(URISyntaxException urisex){
			handler.sendEmptyMessage(RESULTADO_ERROR);
	    }catch (IOException e) {
	    	handler.sendEmptyMessage(RESULTADO_ERROR);
	    }
	}
    
    private int procesarRespuestaRecurosRecibidos (String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;
			
			if (json.has("equipos") && json.has("territorios")){
				int id;
				double equipoLatitud;
				double equipoLongitud;
				int equipoPoseedorId;
				String color;
				List<Equipo> equipos;
				List<Territorio> territorios;
				Equipo equipo;
				Territorio territorio;
				
				JSONArray equiposObject = json.getJSONArray("equipos");
				int longitudEquipos = equiposObject.length()-1;
										
				for (int i = 0; i < longitudEquipos; i++){
					JSONObject JSONEquipo = (JSONObject)equiposObject.get(i);
					
					id = JSONEquipo.getInt("ident");
					equipoLatitud = JSONEquipo.getDouble("latitud");
					equipoLongitud = JSONEquipo.getDouble("longitud");
					
					equipos = PartidaListar.getPartida().getEquipos();
					equipo = Equipo.selecionarPorId(equipos, id);
					equipo.setLatitude(equipoLatitud);
					equipo.setLongitude(equipoLongitud);
				}
				
				JSONArray territoriosObject = json.getJSONArray("territorios");
				int longitudTerritorios = territoriosObject.length()-1;
										
				for (int i = 0; i < longitudTerritorios; i++){
					JSONObject JSONTerritorio = (JSONObject)territoriosObject.get(i);
					
					id = JSONTerritorio.getInt("ident");
					equipoPoseedorId = JSONTerritorio.getInt("equipoId");
					if (equipoPoseedorId < 0)
						color = "gris";
					else
						color = JSONTerritorio.getString("color");
					
					territorios = PartidaListar.getPartida().getTerritorios();
					territorio = Territorio.selecionarPorId(territorios, id);
					territorio.setEquipoPoseedorId(equipoPoseedorId);
					territorio.setColor(color);
				}
			}else{
				return RESULTADO_ERROR;
			}

		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
		return RESULTADO_OK;
	}

}
