package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONObject;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

import com.Conquistadores.App.R;

/**
 * Servicio que se ejecuta peridicamente para actualizar los marcadores de los equipos,
 * sumndoles puntos segn el nmero de territorios que tengan. Tambin se incrementan 
 * las tropas del equipo y de cada territorio que sea dueo.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class ServicioRecursoEquipo extends Service{
	
	static private String TAG = "ServicioRecursoEquipo";
	
	private static final int CRECIMIENTO_BAJO = 250;
	private static final int CRECIMIENTO_MEDIO = 500;
	private static final int CRECIMIENTO_ALTO = 1000;
	private static final int PUNTUACION = 10;
	
	private static final int RETARDO = 180 * 1000;
	private static final int PERIODO = 180 * 1000;
	private static final int RESULTADO_ERROR = -1;
	private static final int RESULTADO_OK = 1;

	private Timer timer;
	private int valorCrecimiento;	
	private String url;
	private List<NameValuePair> datos;	
		
	@Override
	public void onCreate(){
		super.onCreate();
		// Iniciamos el servicio
		this.iniciarServicio();
	}

	@Override
	public void onDestroy(){
		super.onDestroy();
		// Detenemos el servicio
		this.finalizarServicio();
	}
	
	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}

	private void iniciarServicio(){
		Partida partida = PartidaListar.getPartida();
		int partidaId = partida.getIdent();		
		int equipoId = RecursoListar.getEquipo().getIdent();
		String crecimiento = partida.getCrecimiento();
		if(crecimiento.toLowerCase().compareTo("bajo") == 0)
			valorCrecimiento = CRECIMIENTO_BAJO;
		else if(crecimiento.toLowerCase().compareTo("medio") == 0)
			valorCrecimiento = CRECIMIENTO_MEDIO;
		else
			valorCrecimiento = CRECIMIENTO_ALTO;
		String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");		
		url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/servicio/recursos/?format=json",
				urlServer,partidaId,equipoId);
		
		datos = new ArrayList<NameValuePair>();
		BasicNameValuePair bnvp = new BasicNameValuePair("crecimiento", Integer.toString(valorCrecimiento));
		datos.add(bnvp);		
		bnvp = new BasicNameValuePair("puntuacion", Integer.toString(PUNTUACION));
		datos.add(bnvp);	
		
		TimerTask timerTask = new TimerTask(){
			public void run(){
				actualizarRecursoEquipo();
			}
		};
		
		// Creamos el timer
		timer = new Timer();
		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
	}

	private void finalizarServicio(){
		// Detenemos el timer
		if (timer != null)
			this.timer.cancel();
	}
	
    private Handler handler = new Handler(){
    	public void handleMessage(Message msg){
    		int valor = msg.what;
			Context context = getApplicationContext();
			CharSequence text;
			Toast toast;
    		switch(valor){
    		case RESULTADO_ERROR:
    			text = getString(R.string.toastFalloServRecurso);
    			toast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
    			toast.setGravity(Gravity.CENTER, 0, 0);
    			toast.show();
    			break;
    		case RESULTADO_OK:    			
    			text = getString(R.string.toastExitoServRecurso);
    			toast = Toast.makeText(context, text, Toast.LENGTH_LONG);
    			toast.setGravity(Gravity.CENTER, 0, 0);
    			toast.show();
    			break;
    		};
    	}
    };
	
	private void actualizarRecursoEquipo(){
		int resultado;
		try{
			String str = Utilidades.enviarDatos(url, datos);
			resultado = procesarRespuestaRecurosEquipo(str);	
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
			handler.sendEmptyMessage(RESULTADO_ERROR);
	    }catch (IOException e) {
	    	handler.sendEmptyMessage(RESULTADO_ERROR);
	    }
	}
    
    private int procesarRespuestaRecurosEquipo (String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;

		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
		return RESULTADO_OK;
	}

}
