package com.Conquistadores.App;

import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.app.TabActivity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.widget.TabHost;
import android.widget.TextView;

import com.Conquistadores.App.R;

/**
 * Creacin de un TabHost con las principales actividades (Estadstica, Mapa y Mensajera)
 * de la aplicacin.
 * Tambin se establece un Timer junto con un Handler para controlar el tiempo transcurrido de la
 * partida, cuando el reloj llegue a 0 minutos se lanza un intent a la activity estadsticas 
 * para indicar al usuario que la partida ha acabado.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class TabHostPartida extends TabActivity {
	
	private static final int RETARDO = 0;
	private static final int PERIODO = 60 * 1000;	
	private final int RELOJ = 1;
	private final int TERMINAR = 2;
	
	private static boolean TERMINAR_PARTIDA;
	private int tiempoPartida;
	private TextView tiempoRestante;	
	private Timer timer;
	
	public static boolean getTERMINAR_PARTIDA() {
		return TERMINAR_PARTIDA;
	}

	@Override
    protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		final TabHost tabHost = getTabHost();
		
		tabHost.addTab(tabHost.newTabSpec("Estadistica")
                .setIndicator(getString(R.string.estadisticas), this.getResources().getDrawable(R.drawable.ic_tab_estadistica))
                .setContent(new Intent(this, Estadisticas.class)));
		
		tabHost.addTab(tabHost.newTabSpec("Mapa")				
                .setIndicator(getString(R.string.mapa), this.getResources().getDrawable(R.drawable.ic_tab_mapa))
                .setContent(new Intent(this, Mapa.class)));
		
		tabHost.addTab(tabHost.newTabSpec("Mensajes")				
                .setIndicator(getString(R.string.mensajes), this.getResources().getDrawable(R.drawable.ic_tab_msg))
                .setContent(new Intent(this, Mensajeria.class)));
	
		tabHost.computeScroll();
		
		tiempoPartida = PartidaListar.getPartida().getDuracion();
		tiempoRestante = Estadisticas.getTiempoRestante();
		TERMINAR_PARTIDA = false;
		
		// Creamos el timer
		timer = new Timer();
		// Configuramos lo que tiene que hacer
		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
	}
	
	@Override
    protected void onDestroy() {
		super.onDestroy();
		timer.cancel();
	}
	
	private Handler handler = new Handler(){
		@Override
    	public void handleMessage(Message msg){
    		int valor = msg.what;
    		switch(valor){
    		case RELOJ:    			
    			tiempoRestante.setText(String.format(getString(R.string.tiempoRestante), tiempoPartida));
    			tiempoPartida = tiempoPartida-1;
    			break;
    		case TERMINAR:    			
    			Utilidades.finalizarServicios(TabHostPartida.this);
    			Utilidades.finalizarActivities((Activity)TabHostPartida.this);
    			TabHostPartida.TERMINAR_PARTIDA = true;    			
    			Intent intent = new Intent(TabHostPartida.this, Estadisticas.class);
    			TabHostPartida.this.startActivity(intent);
				break;
    		};
    	}
	};
	
	private TimerTask timerTask = new TimerTask(){
		public void run(){			
    		if (tiempoPartida > 0)
    			handler.sendEmptyMessage(RELOJ);
    		else
    			handler.sendEmptyMessage(TERMINAR);   		
		}
	};
}
