package com.Conquistadores.App;

import java.util.ArrayList;
import java.util.List;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.widget.ListAdapter;
import android.widget.Toast;

import com.Conquistadores.App.R;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapView;
import com.google.android.maps.OverlayItem;

/**
 * Clase auxiliar para aadir la capa de Territorios al MapActivity de la aplicacin.
 * Mediante el mtodo draw, representamos los territorios con iconos personalizados
 * segn el equipo poseedor.
 * Si el usuario pulsa sobre un territorio en el mapa, mediante el mtodo onTap, se identifica
 * sobre qu tipo de territorio ha pulsado(sin dueo, propio o rival) y se  muestra su
 * correspondiente context menu.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class TerritorioItemizedOverlay extends ItemizedOverlay<OverlayItem> {

	private Context context;
	private Location localizacionMiEquipo;
	private List<OverlayItem> mOverlays = new ArrayList<OverlayItem>();
	
	public TerritorioItemizedOverlay(Context context, Location localizacion, Drawable defaultMarker) {
		super(boundCenter(defaultMarker));
		this.context = context;
		this.localizacionMiEquipo = localizacion;
	}

	@Override
	protected OverlayItem createItem(int i) {
	  return mOverlays.get(i);
	}
		
    @Override 
    public void draw(Canvas canvas, MapView mapView, boolean shadow){
    	super.draw(canvas, mapView, false);
    }
	
    @Override 
    protected boolean onTap(int indice){
    	final int territorioId = Integer.valueOf(mOverlays.get(indice).getSnippet());
    	List<Territorio> territorios = PartidaListar.getPartida().getTerritorios();
    	Territorio territorio = Territorio.selecionarPorId(territorios, territorioId);
    	if (territorio == null)
    		return false;
    	
    	String titulo;
		final MenuFila[] opciones;
    	int miEquipoId = RecursoListar.getEquipo().getIdent();		
		final int equipoPoseedorId = territorio.getEquipoPoseedorId();
       //Compruebo si el territorio pulsado es de mi propiedad o de otro jugador
		if(equipoPoseedorId == miEquipoId){
    		titulo = context.getString(R.string.menuTerritorio);
        	opciones = new MenuFila[2];
    		opciones[0] = new MenuFila(R.drawable.ic_map_mover, context.getString(R.string.menuMovimiento));
        	opciones[1] = new MenuFila(R.drawable.ic_map_territorio_info, context.getString(R.string.menuInfo));
    	}else{
        	if (equipoPoseedorId == -1){
        		titulo = context.getString(R.string.menuTerritorio);    			
        		opciones = new MenuFila[1];
        		opciones[0] = new MenuFila(R.drawable.ic_map_conquistar, context.getString(R.string.menuConquistar));
        	}else{    		
        		titulo = context.getString(R.string.menuTerritorioRival);
        		opciones = new MenuFila[2];
        		opciones[0] = new MenuFila(R.drawable.ic_map_conquistar, context.getString(R.string.menuConquistar));
        		opciones[1] = new MenuFila(R.drawable.ic_map_espiar, context.getString(R.string.menuEspiar));
        	}
    	}
		
		final GeoPoint puntoTerritorio = mOverlays.get(indice).getPoint();
		ListAdapter adapter = new MenuAdapter(context, opciones);
		AlertDialog.Builder builder = new AlertDialog.Builder(context);
		builder.setAdapter(adapter, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int item) {
				dialog.dismiss();
				String opcion = opciones[item].getTitulo();				
				if(opcion.compareTo(context.getString(R.string.menuConquistar)) == 0){					
					float[] resultados = new float[3];
		    		float distancia;
		    		
		    		Location.distanceBetween(localizacionMiEquipo.getLatitude(), localizacionMiEquipo.getLongitude(), 
		    				puntoTerritorio.getLatitudeE6()/1E6, puntoTerritorio.getLongitudeE6()/1E6, resultados);
		    		distancia = resultados[0];
		    		if (distancia <= Mapa.RADIO_EQUIPO){
		    			OpcionesMenuTerritorio opcionMenu = new OpcionesMenuTerritorio(context, territorioId, equipoPoseedorId);
		    			opcionMenu.conquistar();
		    		}else{
		    			CharSequence text = context.getString(R.string.distanciaConqusitar);
		    			Toast toast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
		    			//toast.setGravity(Gravity.CENTER, 0, 0);
		    			toast.show();
		    		}
				}else if(opcion.compareTo(context.getString(R.string.menuEspiar)) == 0){
					OpcionesMenuTerritorio opcionMenu = new OpcionesMenuTerritorio(context, territorioId);
					opcionMenu.espiar();				
				}else if(opcion.compareTo(context.getString(R.string.menuMovimiento)) == 0){
					OpcionesMenuTerritorio opcionMenu = new OpcionesMenuTerritorio(context, territorioId);
					opcionMenu.mover();	
				}else if(opcion.compareTo(context.getString(R.string.menuInfo)) == 0){
					OpcionesMenuTerritorio opcionMenu = new OpcionesMenuTerritorio(context, territorioId);
					opcionMenu.informacion();
				}
			}
		});
		
		AlertDialog alert = builder.create();
		alert.setTitle(titulo);
		alert.show();		
    	return super.onTap(indice);
    }

	@Override
	public int size() {
		return mOverlays.size();
	}
	
	public void addOverlay(OverlayItem overlay) {
	    mOverlays.add(overlay);
	    populate();
	}

}
