package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONObject;

import com.Conquistadores.App.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Activity lanzada para conquistar un territorio rival. El usuario elige la cantidad
 * de escolta que usar en el ataque.
 * La aplicacin comprueba si la cantidad introducida es correcta, si no lo es, avisa al usuario.
 * Si todo est correcto, se simula la conquista y se muestra el resultado mediante un
 * alert dialog.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class TerritorioRivalConquistar extends Activity implements OnClickListener{
	
	static private String TAG = "TerritorioRivalConquistar";
	
	private final int MENU_AYUDA = Menu.FIRST;
	private final int RESULTADO_ERROR_CONEXION = -3;
	private final int RESULTADO_ERROR = -2;
	private final int VERSION_DIFERENTE = -1;
	private final int RESULTADO_OK = 1;
	private final double PORCENTAJE_MAX_TROPA = 0.95;
	private final double BONIFICACION_TERRITORIO = 0.2;
	private final double PERDIDA_ATACANTE = 0.1;
	private final double PERDIDA_DEFENSOR = 0.1;

	private LinearLayout linearLayout;
	private TextView territorioDefensor;
	private TextView lblTropaEquipo; 
	private TextView tropaNombre;
	private TextView tropaDescrp;
	private EditText campoTropaAtacante;
	private Button botonConquistar;
	
	private int equipoDefensorId;
	private String territorioId;
	private int territorioVersion;
	private String territorioNombre;
	private int tropaDefensivaInic;
	private int tropaAtacanteInicMax;
	private int tropaAtacanteInic;
	private int equipoId;
	private int equipoVersion;
	private int equipoEscolta;	
	private String url;
	
	private ProgressDialog progressDialog;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.territorio_rival_conquistar);
						
		Bundle extras = getIntent().getExtras();
		territorioId = extras.getString("territorioId");
		equipoDefensorId = extras.getInt("equipoDefensorId");
		
		linearLayout = (LinearLayout)findViewById(R.id.linearLayout1);
		linearLayout.setVisibility(View.GONE);		
	    	    
		territorioDefensor = (TextView)this.findViewById(R.id.textTerritorioDefensor);
		lblTropaEquipo = (TextView)this.findViewById(R.id.textLbltropaEquipo);
		tropaNombre = (TextView)this.findViewById(R.id.textTropaNombre);
		tropaDescrp = (TextView)this.findViewById(R.id.textTropaDescrp);
		campoTropaAtacante = (EditText)this.findViewById(R.id.editTropaAtacante);		
		botonConquistar = (Button)this.findViewById(R.id.botonConquistar);		
		botonConquistar.setOnClickListener(this);
	    
		int partidaId = PartidaListar.getPartida().getIdent();
		equipoId = RecursoListar.getEquipo().getIdent();
		String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
		url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/territorio/%4$s/conquistar/?format=json",
				urlServer, partidaId, equipoId, territorioId);
		
		progressDialog = ProgressDialog.show(this, "", 
				getString(R.string.progDialogRecibDatos), true);        
		threadGET.start();
	}
	
	@Override
	protected void onPause(){
		super.onPause();
		finish();
	}
	
	private Thread threadGET = new Thread(){
    	@Override
    	public void run(){
    		Looper.prepare();
    		obtenerDatosConquista();
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
        		int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.errorDatosConquista);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_OK:
    				String s = String.format(getString(R.string.territorioDefensor), territorioNombre);
    				territorioDefensor.setText(s);    				
    				String equipoNombre = RecursoListar.getEquipo().getGroupName();    				
    				s = String.format(getString(R.string.tituloTropaEquipo), equipoNombre);
    				lblTropaEquipo.setText(s);    				
    				s = String.format(getString(R.string.tropaNombre), "Escolta");
    				tropaNombre.setText(s);    				
    				tropaAtacanteInicMax = (int) (equipoEscolta*PORCENTAJE_MAX_TROPA);
    				s = String.format(getString(R.string.tropaDescrp), equipoEscolta, tropaAtacanteInicMax);
    				tropaDescrp.setText(s);	  				

    			    linearLayout.setVisibility(View.VISIBLE);
    			break;
    			}
    		}
    	};
	
    	private void obtenerDatosConquista(){
    		int resultado;
    		
    		try{
    			String str = Utilidades.obtenerDatos(url);
    			resultado = procesarRespuestaDatosConquista(str);
    			handler.sendEmptyMessage(resultado);
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaDatosConquista (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200)
    				return RESULTADO_ERROR; 		 			
    			
    			if (json.has("equipo") && json.has("tropaRival")){   				
    				JSONObject JSONEquipo = json.getJSONObject("equipo");
    				
    				equipoVersion = JSONEquipo.getInt("version");
    				equipoEscolta = JSONEquipo.getInt("escolta");    			
   					
    				JSONArray tropaRivalObject = json.getJSONArray("tropaRival");
					int longitudTropaRival = tropaRivalObject.length()-1;
					for (int j = 0; j < longitudTropaRival; j++){
						JSONObject JSONTropaRival = (JSONObject)tropaRivalObject.get(j);
						
						territorioVersion = JSONTropaRival.getInt("version");
						territorioNombre = JSONTropaRival.getString("territorioNombre");
						tropaDefensivaInic = JSONTropaRival.getInt("cantidad");
					}
    			}else
    				return RESULTADO_ERROR;

    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
	@Override
	public void onClick(View v) {
		if(botonConquistar.isPressed()){
			boolean error = true;
			boolean campoVacio = campoTropaAtacante.getText().toString().trim().equals("");
			if(!campoVacio){
				tropaAtacanteInic =  Integer.valueOf(campoTropaAtacante.getText().toString());
				if (tropaAtacanteInic <= tropaAtacanteInicMax){
					error=false;
					progressDialog = ProgressDialog.show(this, "", 
							getString(R.string.progDialogConquistar), true);        
					threadPOST.start();					
				}
			}
			if(error){
				Context context = getApplicationContext();
				CharSequence text = getString(R.string.numTropaNoValido);
				Toast toast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
				//toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();	
			}
		}
	}
	
	private Thread threadPOST = new Thread(){
    	@Override
    	public void run(){
    		Looper.prepare();
    		simularConquista(); 
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
    			int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.errorDatosConquista);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case VERSION_DIFERENTE:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.errorVersionConquista);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case RESULTADO_OK:
    				Context context = getApplicationContext();
    				CharSequence text;
    				int ganadorId = (Integer)(msg.obj);
    				if(ganadorId == equipoId)
    					text = getString(R.string.exitoConquista);
    				else
    					text = getString(R.string.fracasoConquista);
    				Toast toast = Toast.makeText(context, text, Toast.LENGTH_LONG);
    				toast.show();
    			    TerritorioRivalConquistar.this.setResult(RESULT_OK);
		        	TerritorioRivalConquistar.this.finish();
		        	break;
    			}
    		}
    	};
    	
    	private void simularConquista(){
    		int ganadorId;
    		int tropaDefensivaFinal;
    		int tropaAtacanteFinal;
    		int tropaDefensivaBonificada = (int)(tropaDefensivaInic+(tropaDefensivaInic*BONIFICACION_TERRITORIO));		
    		int tropaTotal = tropaAtacanteInic + tropaDefensivaBonificada;
    		double porcVictAtacante = (double)(tropaAtacanteInic)/tropaTotal;
    		double porcVictDefensor = 1 - porcVictAtacante;
    		
    		//Las tropas defensivas son el doble que las atacantes
    		if(tropaDefensivaBonificada >= (tropaAtacanteInic*2)){
    			ganadorId = equipoDefensorId;
    			tropaDefensivaFinal = (int)(tropaDefensivaInic*porcVictDefensor);
    			tropaAtacanteFinal = 0;
    			//Log.e(TAG, "ATACANTE: "+porcVictAtacante+" DEFENSOR: "+porcVictDefensor+" TROPAS_DEF: "+tropaDefensivaFinal);
    		}else{
    			if(tropaAtacanteInic >= (tropaDefensivaBonificada*2)){
    				ganadorId = equipoId;
        			tropaDefensivaFinal = 0;
        			tropaAtacanteFinal = (int)(tropaAtacanteInic*porcVictAtacante);
        			//Log.e(TAG, "ATACANTE: "+porcVictAtacante+" DEFENSOR: "+porcVictDefensor+" TROPA_ATAC: "+tropaAtacanteFinal);
    			}else{
        			double porcBatalla = Math.random();
        			//El equipo atacante conquista el territorio
        			if (porcVictAtacante > porcBatalla)
        				ganadorId = equipoId;
        			else
        				ganadorId = equipoDefensorId;
        			
        			double rango = PERDIDA_ATACANTE / 2;
        			double limiteInferior = porcVictAtacante - rango;
        			double limiteSuperior = porcVictAtacante + rango;      			
        			double aleatoriedad = Math.random()*(limiteSuperior-limiteInferior)+(limiteInferior);
        			tropaAtacanteFinal = (int)(tropaAtacanteInic*aleatoriedad);
        			//Log.e(TAG, "ATACANTE: "+porcVictAtacante+" DEFENSOR: "+porcVictDefensor+" TIRADA:"+porcBatalla+" %TROPAS_ATAC: "+aleatoriedad);

        			rango = PERDIDA_DEFENSOR / 2;
        			limiteInferior = porcVictDefensor - rango;
        			limiteSuperior = porcVictDefensor + rango;      			
        			aleatoriedad = Math.random()*(limiteSuperior-limiteInferior)+(limiteInferior);
        			tropaDefensivaFinal = (int)(tropaDefensivaInic*aleatoriedad);
        			//Log.e(TAG, "ATACANTE: "+porcVictAtacante+" DEFENSOR: "+porcVictDefensor+" TIRADA:"+porcBatalla+" %TROPAS_DEF: "+aleatoriedad);
    			}
    		}	
    		EnviarDatosConquista(ganadorId, tropaDefensivaFinal, tropaAtacanteFinal);	
    	}
	
    	private void EnviarDatosConquista(int ganadorId, int tropaDefensivaFinal, int tropaAtacanteFinal){
    		int resultado;
    		
    		try{
    			List<NameValuePair> datos = new ArrayList<NameValuePair>();
    			BasicNameValuePair bnvp = new BasicNameValuePair("ganadorId", Integer.toString(ganadorId));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("equipoAtacanteVersion", Integer.toString(equipoVersion));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("equipoDefensorId", Integer.toString(equipoDefensorId));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("territorioVersion", Integer.toString(territorioVersion));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("tropaDefensivaFinal", Integer.toString(tropaDefensivaFinal));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("tropaAtacanteInic", Integer.toString(tropaAtacanteInic));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("tropaAtacanteFinal", Integer.toString(tropaAtacanteFinal));
    			datos.add(bnvp);			
    			bnvp = new BasicNameValuePair("puntuacion", Integer.toString(OpcionesMenuTerritorio.PUNTOS_CONQUISTA));
    			datos.add(bnvp);
    			String str = Utilidades.enviarDatos(url, datos);
    			resultado = procesarRespuestaEnviarConquista(str);
    			  			
    			Message message = Message.obtain();
    			message.obj = ganadorId;
    			message.what = resultado;
    			handler.sendMessage(message);
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaEnviarConquista (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200){
    				if (info.toLowerCase().compareTo("version diferente") == 0)
    					return VERSION_DIFERENTE;
    				return RESULTADO_ERROR;
    			}

    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
	private void showAlert(int icono, String titulo, String msg){ 	
		AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   dialog.cancel();
		        	   TerritorioRivalConquistar.this.setResult(RESULT_OK);
		        	   TerritorioRivalConquistar.this.finish();
    	           }
    	       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
    }
    
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaConquistar);
    		showAlertAyuda(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }

	private void showAlertAyuda(int icono, String titulo, String msg){ 	
		AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   dialog.cancel();
    	           }
    	       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
    }
	
}
