package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONObject;

import com.Conquistadores.App.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

/**
 * Activity lanzada para mover tropas entre territorios o equipo. En primer lugar, el usuario
 * elige el origen de donde se van a obtener las tropas. Luego elige la cantidad de unidades
 * que se sumarn a las tropas del destino.
 * La aplicacin comprueba si la cantidad introducida es correcta, si no lo es, avisa al usuario.
 * Si todo est correcto, realiza el movimiento y se muestra el resultado mediante un
 * toast.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class TropaMover extends Activity implements OnClickListener{
	
	static private String TAG = "TerritorioMoverTropa";
	
	private final int MENU_AYUDA = Menu.FIRST;
	private final int RESULTADO_ERROR_CONEXION = -5;
	private final int RESULTADO_ERROR = -4;	
	private final int VERSION_DIFERENTE = -3;
	private final int RESULTADO_NO_TROPAS = -2;
	private final int RESULTADO_OK = 1;
	private final double PORCENTAJE_MAX_TROPA = 0.95;
	
	private LinearLayout linearLayout;	
	private Spinner spinner;
	private TextView destino;
	private TextView lblTropaOrigen; 
	private TextView tropaNombre;
	private TextView tropaDescrp;
	private EditText campoTropaMover;
	private Button botonMover;
	
	private List<Integer> listVersion;
	private List<Integer> listId;
	private List<String> listNombre;
	private List<String> listTropaNombre;
	private List<Integer> listTropaCantidad;
	
	private String tipo_origen;
	private String tipo_destino;
	private int origenId;
	private int origenVersion;
	private int tropaOrigenMax;
	private int numTropaOrigen;
	private int destinoId;
	private int destinoVersion;
	private String destinoNombre;
	private String destinoTropaNombre;
	private int destinoTropaCantidad;
	private String url;
	
	private ProgressDialog progressDialog;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.tropa_mover);
						
		Bundle extras = getIntent().getExtras();
		tipo_destino = extras.getString("tipo_destino");
		destinoId = extras.getInt("destinoId");			
		
		linearLayout = (LinearLayout)findViewById(R.id.linearLayout1);
		linearLayout.setVisibility(View.GONE);
		
		spinner = (Spinner)this.findViewById(R.id.spinnerOrigen);		
		destino = (TextView)this.findViewById(R.id.textDestino);	    	    
		lblTropaOrigen = (TextView)this.findViewById(R.id.textLblTropaOrigen);
		tropaNombre = (TextView)this.findViewById(R.id.textTropaNombre);
		tropaDescrp = (TextView)this.findViewById(R.id.textTropaDescrp);
		campoTropaMover = (EditText)this.findViewById(R.id.editTropaMover);		
		botonMover = (Button)this.findViewById(R.id.botonMover);		
		botonMover.setOnClickListener(this);
		
		String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");
		int partidaId = PartidaListar.getPartida().getIdent();
		int equipoId = RecursoListar.getEquipo().getIdent();
		url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/%4$s/%5$s/tropa/mover/?format=json",
				urlServer, partidaId, equipoId, tipo_destino, destinoId);
	    
		progressDialog = ProgressDialog.show(this, "", 
				getString(R.string.progDialogRecibDatos), true);        
		threadGET.start();
	}
	
	@Override
	protected void onPause(){
		super.onPause();
		finish();
	}

	private Thread threadGET = new Thread(){
    	@Override
    	public void run(){
    		Looper.prepare();
    		obtenerDatosTropaMovimiento();
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
    			int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();
    			switch(valor){
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.errorDatosMovimiento);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_NO_TROPAS:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.noTerritorios);
    				showAlert(icono,titulo,msgAlert);
    			break;
    			case RESULTADO_OK:
    				ArrayAdapter<String> adapter = new ArrayAdapter<String>(getApplicationContext(), 
    			    		android.R.layout.simple_spinner_item, listNombre);
    			    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    			    spinner.setAdapter(adapter);
    			    
    				String s = String.format(getString(R.string.destino), destinoNombre,destinoTropaNombre,destinoTropaCantidad);
    				destino.setText(s);
    				
    			    MyOnItemSelectedListener listener = new MyOnItemSelectedListener();
    			    spinner.setOnItemSelectedListener(listener);
    			    linearLayout.setVisibility(View.VISIBLE);
    			break;
    			}
    		}
    	};
	
    	private void obtenerDatosTropaMovimiento(){
    		int resultado;
    		
    		try{
    			String str = Utilidades.obtenerDatos(url);
    			resultado = procesarRespuestaDatos(str);
    			handler.sendEmptyMessage(resultado);

    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuestaDatos (String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200)
    				return RESULTADO_ERROR;
    			
				listVersion = new ArrayList<Integer>();
				listId = new ArrayList<Integer>();
				listNombre = new ArrayList<String>();
				listTropaNombre = new ArrayList<String>();
				listTropaCantidad = new ArrayList<Integer>();   			
    			
    			if (json.has("tropas")){
    				int territorioId;
    				int territorioVersion;
    				String territorioNombre;
    				String tropaNombre;
    				int tropaCantidad;
    				int equipoId;
    				int equipoVersion;
    				
    				JSONArray tropasObject = json.getJSONArray("tropas");
    				int longitud = tropasObject.length()-1;    					
    				for (int i = 0; i < longitud; i++){
    					JSONObject JSONTropa = (JSONObject)tropasObject.get(i);
    					
    					territorioId = JSONTropa.getInt("ident");
    					territorioVersion = JSONTropa.getInt("version");
    					territorioNombre = JSONTropa.getString("nombre");
    					tropaNombre = JSONTropa.getString("tipo");
    					tropaCantidad = JSONTropa.getInt("cantidad");    					
        				
    					if(destinoId != territorioId){
    						listVersion.add(territorioVersion);
    						listId.add(territorioId);
    						listNombre.add(territorioNombre);
    						listTropaNombre.add(tropaNombre);
    						listTropaCantidad.add(tropaCantidad);
    					}else{
							destinoVersion=territorioVersion;
							destinoNombre=territorioNombre;
							destinoTropaNombre=tropaNombre;
							destinoTropaCantidad=tropaCantidad;
    					}
    				}
    				JSONArray EquipoObject = json.getJSONArray("equipo");
					int longitudEquipo = EquipoObject.length()-1;
					for (int j = 0; j < longitudEquipo; j++){
						JSONObject JSONEquipo = (JSONObject)EquipoObject.get(j);
							
						equipoId = JSONEquipo.getInt("ident");
						equipoVersion = JSONEquipo.getInt("version");
						tropaCantidad = JSONEquipo.getInt("escolta");
						
						if(destinoId != equipoId){
							listVersion.add(equipoVersion);
							listId.add(equipoId);
							listNombre.add("Equipo");
							listTropaNombre.add("Escolta");
							listTropaCantidad.add(tropaCantidad);
						}else{
							destinoVersion=equipoVersion;
							destinoNombre="Equipo";
							destinoTropaNombre="Escolta";
							destinoTropaCantidad=tropaCantidad;
						}
					}
    			}else
    				return RESULTADO_NO_TROPAS;

    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
	@Override
	public void onClick(View v) {
		if(botonMover.isPressed()){
			boolean error = true;
			boolean campoVacio = campoTropaMover.getText().toString().trim().equals("");
			if(!campoVacio){
				numTropaOrigen =  Integer.valueOf(campoTropaMover.getText().toString());
				if (numTropaOrigen <= tropaOrigenMax){
					error=false;
					progressDialog = ProgressDialog.show(this, "", 
							getString(R.string.progDialogMoverTropas), true);        
					threadPOST.start();					
				}
			}
			if(error){
				Context context = getApplicationContext();
				CharSequence text = getString(R.string.numTropaNoValido);
				Toast toast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
				//toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
			}
		}
	}
	
	private Thread threadPOST = new Thread(){
    	@Override
    	public void run(){
    		Looper.prepare();
    		EnviarDatos(); 
    		Looper.myLooper().quit();
    		Looper.loop();
    	}
    	
    	private Handler handler = new Handler(){
    		@Override
    		public void handleMessage(Message msg){
        		int valor = msg.what;
    			int icono;
    			String titulo;
    			String msgAlert;
    			
    			progressDialog.cancel();    			
    			switch(valor){		
    			case RESULTADO_ERROR_CONEXION:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.falloConexionServer);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case RESULTADO_ERROR:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.errorDatosMovimiento);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case VERSION_DIFERENTE:
    				icono = R.drawable.ic_dialog_warning;
    				titulo = getString(R.string.aviso);
    				msgAlert = getString(R.string.errorVersionMovimiento);
    				showAlert(icono,titulo,msgAlert);
    				break;
    			case RESULTADO_OK:
    				Context context = getApplicationContext();
    				CharSequence text = getString(R.string.exitoMovimiento);
    				Toast toast = Toast.makeText(context, text, Toast.LENGTH_LONG);
    				toast.show();
    				TropaMover.this.setResult(RESULT_OK);
    	        	TropaMover.this.finish();
		        	break;
    			}
    		}
    	};

    	private void EnviarDatos(){
    		int resultado;
    		
    		try{
    			List<NameValuePair> datos = new ArrayList<NameValuePair>();

    			BasicNameValuePair bnvp = new BasicNameValuePair("origenId", Integer.toString(origenId));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("origenVersion", Integer.toString(origenVersion));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("destinoVersion", Integer.toString(destinoVersion));
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("tipoOrigen", tipo_origen);
    			datos.add(bnvp);
    			bnvp = new BasicNameValuePair("numTropaOrigen", Integer.toString(numTropaOrigen));
    			datos.add(bnvp);			

    			String str = Utilidades.enviarDatos(url, datos);
    			resultado = procesarRespuesta(str);
    			
    			handler.sendEmptyMessage(resultado);    			
    			
    		}catch(URISyntaxException urisex){
    		    Log.e(TAG, urisex.getMessage());
    		    handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }catch (IOException e) {
                Log.e(TAG, e.getMessage());
                handler.sendEmptyMessage(RESULTADO_ERROR_CONEXION);
    	    }
    	}

        private int procesarRespuesta(String str) {
        	int code;
        	String info;
            try {
            	JSONObject json;
            	json = new JSONObject(str.toString());

    			if (json.has("codigo"))
    				code = json.getInt("codigo");
    			else
    				code = -1;

    			if (json.has("info"))
    				info = json.getString("info");
    			else
    				info = null;
    			
    			if (code != 200){
    				if (info.toLowerCase().compareTo("version diferente") == 0)
    					return VERSION_DIFERENTE;
    				return RESULTADO_ERROR;
    			}

    		} catch (Exception e) {	
    			code = 500;
    			info = "Error en el parsed json: " + e.getMessage();
    			Log.e(TAG, info);
    			return RESULTADO_ERROR;
    		}
            return RESULTADO_OK;
    	}      
    };
    
	private void showAlert(int icono, String titulo, String msg){ 	
		AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   dialog.cancel();
		        	   TropaMover.this.setResult(RESULT_OK);
		        	   TropaMover.this.finish();
    	           }
    	       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
    }
    
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(0, MENU_AYUDA, 0, R.string.ayuda).setIcon(R.drawable.ic_menu_help);
    	return true;
    }
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_AYUDA:
        	int icono = R.drawable.ic_dialog_help;
    		String titulo = getString(R.string.ayuda);
    		String msg = getString(R.string.ayudaMovimiento);
    		showAlertAyuda(icono, titulo, msg);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
	private void showAlertAyuda(int icono, String titulo, String msg){ 	
		AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setMessage(msg).setCancelable(false)
		       .setPositiveButton(getString(R.string.botonAceptar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   dialog.cancel();
		        	   TropaMover.this.setResult(RESULT_OK);
		        	   TropaMover.this.finish();
    	           }
    	       });
		AlertDialog alert = adb.create();
		alert.setTitle(titulo);
		alert.setIcon(icono);
		alert.show();
    }
	
	public class MyOnItemSelectedListener implements OnItemSelectedListener {
		public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {			
			String nombre = listNombre.get(pos);
			String s = String.format(getString(R.string.tituloTropaOrigen), nombre);
			lblTropaOrigen.setText(s);
			if(nombre.toLowerCase().compareTo("equipo") == 0)
				tipo_origen="equipo";
			else
				tipo_origen="territorio";
			s = String.format(getString(R.string.tropaNombre), listTropaNombre.get(pos));
			tropaNombre.setText(s);
			
			tropaOrigenMax = (int) (listTropaCantidad.get(pos)*PORCENTAJE_MAX_TROPA);
			s = String.format(getString(R.string.tropaDescrp), listTropaCantidad.get(pos),tropaOrigenMax);
			tropaDescrp.setText(s);	
			origenId = listId.get(pos);
			origenVersion = listVersion.get(pos);
		}
		
		public void onNothingSelected(AdapterView<?> parent) {
		}
	}

}
