package com.Conquistadores.App;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.HTTP;

import com.LibreGeoSocial.Api.LibreGeoSocial;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.view.ViewGroup;
import android.widget.ListAdapter;
import android.widget.ListView;

/**
 * Clase con mtodos auxiliares utilizados por otras clases del proyecto.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class Utilidades{
	
	public static boolean estadoInternet(Context context) {
		ConnectivityManager cm = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo ni = null;
		try{
			ni = cm.getActiveNetworkInfo();
			return ni.isConnectedOrConnecting();
		}catch(NullPointerException npe){
			return false;
		}
	}
	
	public static boolean estadoGPS(Context context) {
		LocationManager lm = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
		try{
			return lm.isProviderEnabled(LocationManager.GPS_PROVIDER);
		}catch(Exception e){
			return false;
		}
	}
	
	public static String obtenerDatos(String url) throws URISyntaxException, IllegalStateException, IOException{
		DefaultHttpClient httpclient = new DefaultHttpClient();
		URI uri = new URI(url);
		HttpGet httpget = new HttpGet(uri);
		LibreGeoSocial LGS = LibreGeoSocial.getInstance();
		httpclient.setCookieStore(LGS.getCookieStore());
		HttpResponse response = httpclient.execute(httpget);
		HttpEntity entity = response.getEntity();
		String str = Utilidades.convertirStreamAString(entity.getContent());
		return str;
	}
	
	public static String enviarDatos(String url, List<NameValuePair> datos) throws URISyntaxException, IllegalStateException, IOException{
		DefaultHttpClient httpclient = new DefaultHttpClient();
		URI uri = new URI(url);
		HttpPost httppost = new HttpPost(uri);
		LibreGeoSocial LGS = LibreGeoSocial.getInstance();
		httpclient.setCookieStore(LGS.getCookieStore());		
		httppost.setEntity(new UrlEncodedFormEntity(datos, HTTP.UTF_8));
		HttpResponse response = httpclient.execute(httppost);
		HttpEntity entity = response.getEntity();
		String str = Utilidades.convertirStreamAString(entity.getContent());
		return str;
	}
	
	public static String convertirStreamAString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String linea = null;
        try {
            while ((linea = reader.readLine()) != null) {
                sb.append(linea + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
	
    public static void setListViewHeightBasedOnChildren(ListView listView, float density) {
        ListAdapter listAdapter = listView.getAdapter();
        if (listAdapter == null) {
            // pre-condition
            return;
        }

        int totalHeight = 0;

        for (int i = 0; i < listAdapter.getCount(); i++) {
        	totalHeight += 25;
        }
        totalHeight = (int) (totalHeight * density);
        ViewGroup.LayoutParams params = listView.getLayoutParams();
        params.height = totalHeight + (listView.getDividerHeight() * (listAdapter.getCount() - 1));
        listView.setLayoutParams(params);
        listView.requestLayout();
    }
    
    public static void finalizarServicios(Context context){
		Intent intent = new Intent(context, ServicioLocalizacion.class);
		context.stopService(intent);
		intent = new Intent(context, ServicioRecursoEquipo.class);
		context.stopService(intent);
		intent = new Intent(context, ServicioMensajeria.class);
		context.stopService(intent);
		intent = new Intent(context, ServicioEnfrentamiento.class);
		context.stopService(intent);
    }
    
    public static void finalizarActivities(Activity actual){
        boolean flag = true;
        Activity inferior = actual.getParent();
        actual.finish();
        if (inferior == null)
            return;
        while (flag) {
            Activity aux = inferior;
            try {
            	inferior = aux.getParent();
            	aux.finish();
            } catch (Exception e) {
                flag = false;
            }
        }
    }
}

