/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Authors : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  		  Juan Francisco Gato Luis <jfcogato@gsyc.es>
 *
 */

package com.LibreGeoSocial.Api;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.location.Location;
import android.util.Log;

import com.Conquistadores.App.R;
import com.LibreGeoSocial.Api.Layers.Category;
import com.LibreGeoSocial.Api.Layers.GenericLayer;
import com.LibreGeoSocial.App.Routes.LocationService;
import com.LibreGeoSocial.App.Utils.BitmapUtils;
import com.LibreGeoSocial.App.Utils.ClientHttpRequest;


public class LibreGeoSocial {

	
	private static LGSCookie sessionCookie;
	
	private static TrustManager[] trustManagers;
	
	/* TAG for log messages*/
	static private String TAG = "LibreGeoSocial";
	
	/* Singleton */
	static private LibreGeoSocial singleton = null;
	
	/* Format that server answer*/
	private String mFormat = "JSON";
	
	/* Url from server: Ex: test.libregeosocial.libresoft.es/social/ */
	private String mUrl = null;
	
	private String mUrlHttps = null;
	
	/* Cookies for connection */
	private LGSCookieStore cookieStore;
	
	/* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	/* Id for last node uploaded */
	private Integer lastId = 0;
	
	private User mUser = null;
	private boolean loged = false;
	
	/* Photos paginacion */
	private static Integer mpagePhoto = 1;
	
	/* Notes pagination */
	private static Integer mpageNote = 1;
	
	
	private ArrayList<GeoNode> mUserList = null;
	
	private ArrayList<GeoNode> mFriendList = null;
	
	public void resetUser(){
		mUser = null;
		loged = false;
	}
	
	static public LibreGeoSocial getInstance() 
	{
        if (singleton == null) {
            singleton = new LibreGeoSocial();
        }
        return singleton;
		
	}
	
	public void setFormat (String format)
	{
		
		if ( format.compareTo("JSON") != 0 && format.compareTo("XML") != 0) 
		{
			Log.e("TAG", "Unknow format\n");
		}
		
		mFormat = format;
		
	}
	
	// jfernandez: getCookieStore()
	public LGSCookieStore getCookieStore(){
		return cookieStore;
	}
	
	public String getUrlServer ()
	{
		return mUrl;
	}
	
	public void setUrl (String Url)
	{
		mUrl = "http://" + Url;
		mUrlHttps = "https://" + Url;
	}
	
	public String getLastMsgServer()
	{		
		return "[" + lastCode.toString() + "]: " + lastMsg;
	}
	
	public Integer getLastId ()
	{
		return lastId;
	}
	
	
    /**
     * Do user register in the server.
     * 
     * @param User object with info user.
     * 
     * @return Boolean variable operation
     */
	
	public boolean registerUser (User user)
	{
		try {            
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			HttpPost httpost = new HttpPost(mUrl + "user/create/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("username", user.getUsername()));		
		    nvps.add(new BasicNameValuePair("password", user.getPassword()));
		    nvps.add(new BasicNameValuePair("first_name", user.getName()));
		    nvps.add(new BasicNameValuePair("last_name", "" ));
		    nvps.add(new BasicNameValuePair("email", user.getEmail() ));
		    nvps.add(new BasicNameValuePair("latitude", user.getLatitude().toString() ));
		    nvps.add(new BasicNameValuePair("longitude", user.getLongitude().toString() ));
		    nvps.add(new BasicNameValuePair("radius", user.getRadius().toString() ));
		    nvps.add(new BasicNameValuePair("country", user.getCountry() ));
		    //nvps.add(new BasicNameValuePair("post_code", user.getPostCode().toString() ));
		    //nvps.add(new BasicNameValuePair("birthday", user.getBirthday() ));
		    
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
    
		    HttpResponse response = httpclient.execute(httpost);
		    
		    HttpEntity entity = response.getEntity(); 
		    
            String str = convertStreamToString(entity.getContent());
            
            
            return processMessage(str);
            
		} catch (IOException e) {
	        // TODO Auto-generated catch block
	        Log.e(TAG,e.getMessage());
	        return false;
		}
	
	}
	
	
    /**
     * Do login with server. If login is ok, the session cookie is saved
     * 
     * @param username The username
     * @param password The password
     * 
     * @return Boolean
     */
	
	public boolean login (String username, String password) 
	{
		//jfernandez - HttpsURLConnection urlConn;
		HttpURLConnection urlConn;
		URL url;
		
        try {      
        	//jfernandez - String secureURL = mUrlHttps + "user/login/?format=JSON";
        	String secureURL = mUrl + "user/login/?format=JSON";

        	//jfernandez - allowAllSSL();
        	url = new URL(secureURL);

        	//jfernandez - urlConn = (HttpsURLConnection) url.openConnection();
			urlConn = (HttpURLConnection) url.openConnection();
			
			// set Post method
	        urlConn.setDoOutput (true);
	        
	        // setting headers
	        urlConn.setRequestMethod("POST");
	        urlConn.setRequestProperty("Content-type", "application/x-www-form-urlencoded");       
	        urlConn.setRequestProperty("Accept-Encoding", "gzip, deflate");
	        urlConn.setRequestProperty("Host", "login.facebook.com");  
            urlConn.setRequestProperty("Connection", "Keep-Alive");
			
			// making content
	        String data = "username=" + username + 
	        				"&password="+ password;

	        // writing content
			OutputStreamWriter out = new OutputStreamWriter(urlConn.getOutputStream());
			out.write (data);
			out.flush();
			
			// getting response
			
			InputStream in = urlConn.getInputStream();
						
			String str = convertStreamToString(in);
			
			Log.e(TAG,str);

			if ( processMessage(str) )
			{
				String headerName;
				
				/* Search cookie */
				for (int i=1; (headerName = urlConn.getHeaderFieldKey(i))!=null; i++) {
					if (headerName.equals("set-cookie")) { 
						
						/* Parse the cookie */
						String cookieStr = urlConn.getHeaderField(i);

						cookieStr = cookieStr.substring(0, cookieStr.indexOf(";"));
					    String cookieName = cookieStr.substring(0, cookieStr.indexOf("="));
					    String cookieValue = cookieStr.substring(cookieStr.indexOf("=") + 1, cookieStr.length()); 
					    
					    /* Generate our cookie */
			        	
					    sessionCookie = new LGSCookie();					    
					    //sessionCookie.setDomain(mUrl.split("/")[2]);
					    //sessionCookie.setDomain("10.0.2.2");
					    sessionCookie.setDomain("192.168.0.198");
					    sessionCookie.setName(cookieName);
					    sessionCookie.setValue(cookieValue);
					    
					    cookieStore = new LGSCookieStore();
					    cookieStore.addCookie(sessionCookie);
					    
					}
				}
				//we put the loged true and make take the user info to be storedn in mUser
				loged = true;
				getInfoUser(true, null);
				return true;
			}
            else
            	return false;
        	
		    
		} catch (IOException e) {
		        // TODO Auto-generated catch block
 		        Log.e(TAG,e.getMessage());
		        return false;
		}
		
	}

	public Location getLocation(){
		
		Location myLocation = LocationService.getCurrentLocation();
		
		if ((myLocation == null) & (isLogged())){
			LocationService.setLastKnownLocation(mUser.getLocation());
			myLocation = mUser.getLocation();
		}
		
		return myLocation;
	}
	
	public ArrayList<GeoNode> getLayerNodes (Integer id, String pattern, String category, Double distance, Integer page, Integer elems)
	{
	
		
		try
		{
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			HttpPost httpost;
		

			if (pattern==null)
				pattern = "";
			
			if (getLocation() == null){
				Log.e("getLocation", "Not location returnet");
				return null;
			}
			
			httpost = new HttpPost(mUrl +  "layer/" + id.toString() + 
								   "/search/?search=" + pattern + 
								   "&category=" + category +
								   "&latitude=" + String.valueOf(getLocation().getLatitude()) + 
								   "&longitude=" + String.valueOf(getLocation().getLongitude()) + 
								   "&page=" + page.toString() +
								   "&elems=" + elems.toString() +
								   "&radius=" + distance.toString() + "&format=JSON");
							
			Log.e("LGS-getLayerNodes:", mUrl +  "layer/" + id.toString() + 
								   "/search/?search=" + pattern + 
								   "&category=" + category +
								   "&latitude=" + getLocation().getLatitude() + 
								   "&longitude=" + getLocation().getLongitude() + 
								   "&page=" + page.toString() +
								   "&elems=" + elems.toString() +
								   "&radius=" + distance.toString() + "&format=JSON");
		
		
			if (isLogged())
				httpclient.setCookieStore(cookieStore);
						
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
	        ArrayList<GeoNode> array = new ArrayList<GeoNode>();
	        ArrayList<GeoNode> aux = null;
	        
	        aux = parseNodes (str);
	        if (aux!=null)
	        	array.addAll (aux);
	        
	        Log.e("LGS: size array=",String.valueOf(array.size()));
	 
	        return array;
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
	}
	
	

    /**
     * Return all users in the server
     * 
     * @param reaload If is false, data obtains of cache.
     * @param distance The maximum distance of users
     * 
     * @return The user list in array list.
     */
	
	public ArrayList<GeoNode> getUserList (boolean reload, Double distance, String pattern, Integer page)
	{
	
		if (reload == false && mUserList != null)
			return mUserList;
		
		try
		{
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			HttpPost httpost;
			
			if (distance == 0.0)
				 httpost = new HttpPost(mUrl + "user/list/?page=" + page.toString() + "&format=JSON"); 
			else
			{
				if (pattern==null)
					pattern = "";
				httpost = new HttpPost(mUrl +  "search/?terms=" + pattern + "&dist=" + distance.toString() + "&models=person&format=JSON"); 
			}
			if (isLogged())
				httpclient.setCookieStore(cookieStore);
						

			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
	        mFriendList = parseNodes(str);
	        return mFriendList;
			
        
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
	}
	
	
    /**
     * Return all friends of login user.
     * 
     * @param reaload If is false, data obtains of cache.
     * 
     * @return The friend list in array list.
     */
	
	public ArrayList<GeoNode> getFriendsList (boolean reload, Double distance, Integer page)
	{
		if (isLogged() == false)
			return null;
		
		if (reload == false && mFriendList != null)
			return mFriendList;
		
		try
		{
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			Log.e("URL", mUrl + "user/" + mUser.getId().toString() + "/friends/?page=" + page.toString() +  "&format=JSON");
	
			HttpPost httpost = new HttpPost(mUrl + "user/" + mUser.getId().toString() + "/friends/?page=" + page.toString() +  "&format=JSON"); 
			
			nvps.add(new BasicNameValuePair("elems", String.valueOf(15)));
			httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
        
	        
	        mFriendList = parseNodes(str);
	        return mFriendList;
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		

	}

	
    /**
     * Return the GeoNode updated
     * @param node The node we want updated
     * 
     * @return The GeoNode
     */
	public GeoNode updateNode (GeoNode node)
	{
		/*
		String token = null;
		
		if ( Photo.class.isInstance(node) )    	
			token = "photo";
		else if (Note.class.isInstance(node) )
			token = "note";
		else if (User.class.isInstance(node) )
			token = "user";
		else if (Audio.class.isInstance(node) )
			token = "audio";
		*/
		try
		{			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			Log.e("id", node.getId().toString());
			
			Integer nodeId = node.getId();
			
			if (nodeId == 0){
				return node;
			}
			
			Integer layerId = (Integer) getLayerId(nodeId).get(0);
			
			Log.e("URL", mUrl + "layer/" + layerId.toString() +"/node/"+ node.getId().toString() + "/data/?format=JSON");
			
			HttpGet httpGet = new HttpGet(mUrl + "layer/" + layerId.toString() +"/node/"+ node.getId().toString() + "/data/?format=JSON");						
			
			if (isLogged())
				httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpGet);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);

	    
	        
	        ArrayList<GeoNode> list = new ArrayList<GeoNode>();
	        
	        list = parseNodes(str);
	        
	        if (list == null)
	        	return null;
	        
            return list.get(0);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
	}

	
    /**
     * Return the friendship invitations
     *  
     * 
     * @return The user info
     */
	
	public ArrayList<GeoNode> getFrienshipInvitations ()
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpGet httpGet = new HttpGet(mUrl + "user/friendship_invitations/?format=JSON");						
						  
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpGet);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return parseNodes(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
		
	}
	
	
    /**
     * Return the info user
     * 
     * @param reaload If is false, data obtains of cache.
     * @param userId The identifier of user 
     * 
     * @return The user info
     */
	
	public User getInfoUser (boolean reload, Integer userId)
	{
		if (isLogged()){
			if ( reload == false && mUser != null)
				return mUser;
		
			try
			{
				DefaultHttpClient httpclient = new DefaultHttpClient();
			
				HttpPost httpget = null;
			
				if (userId==null)
					httpget = new HttpPost(mUrl + "user/data/?format=JSON");
				else
					httpget = new HttpPost(mUrl + "user/" + userId.toString() + "/data/?format=JSON");
			
				httpclient.setCookieStore(cookieStore);
				HttpResponse response = httpclient.execute(httpget);
	
				HttpEntity entity = response.getEntity(); 
		    
				String str = convertStreamToString(entity.getContent());	        
	        
				Log.e(TAG, str);
	        
           
				ArrayList<GeoNode> list = new ArrayList<GeoNode>();
	        
				Log.e("ARRAY", str);
				list = parseNodes(str);
	        
				if (list != null){
					mUser = (User) list.get(0);
					return mUser;
				}
	        
				//If list is null we return null
				return null;
        
			}
		
			catch (IOException e) {
				Log.e(TAG,e.getMessage());
				return null;
			}
		} else { 
			Log.e("InfoUser", "NULL");
			return null;
		}
		
	}
	
	public boolean setAvatar( Integer photoId )
	{
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/set_avatar/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("photo_id", photoId.toString() )); 
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 		    
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return processMessage(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
	}
	
    /**
     * Set the status of user.
     * 
     * @param status Text of the status
     * 
     * @return Boolean
     */
	
	public boolean setStatus (String status)
	{
	
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/set_status/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("status", status )); 
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 		    
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return processMessage(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
	}
	
    /**
     * Set friend of the user
     * 
     * @param idFriend Identifier of friend
     * 
     * @return Boolean variable
     */
	
	public boolean setFriend (Integer idFriend)
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/relation/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("userid1", mUser.getId().toString() )); 
		    nvps.add(new BasicNameValuePair("userid2", idFriend.toString() ));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 		    
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return processMessage(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
		
	}
	
    /**
     * Delete a relation of friend
     * 
     * @param idFriend Identifier of friend
     * 
     * @return Boolean variable
     */
	public boolean deleteFriend (Integer idFriend)
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/relation/delete/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("userid1", mUser.getId().toString() )); 
		    nvps.add(new BasicNameValuePair("userid2", idFriend.toString() ));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 		    
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return processMessage(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
		
	}
	
    /**
     * Set an user geo-position 
     * 
     * @param latitude latitude coordenate
     * @param longitude longitude coordenate
     * 
     * @return Boolean variable
     */
	public boolean setUserPosition (Double latitude, Double longitude, Double altitude)
	{
		if (isLogged()){
			try
			{
				DefaultHttpClient httpclient = new DefaultHttpClient();
			
				HttpPost httpost = new HttpPost(mUrl + "user/position/?format=JSON"); 
			
				List <NameValuePair> nvps = new ArrayList <NameValuePair>();
				nvps.add(new BasicNameValuePair("latitude", latitude.toString()));
				nvps.add(new BasicNameValuePair("longitude", longitude.toString()));
				nvps.add(new BasicNameValuePair("altitude", altitude.toString()));
			
				httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
		    
				httpclient.setCookieStore(cookieStore);
				HttpResponse response = httpclient.execute(httpost);
	
				HttpEntity entity = response.getEntity(); 
		    
				String str = convertStreamToString(entity.getContent());	        
	        
				Log.e(TAG, str);
	        
				return processMessage(str);
	                
			}
		
			catch (IOException e) {
				Log.e(TAG,e.getMessage());
				return false;
			}
		} else {
			return false;
		}
		
	}
	
    /**
     * Create and upload note to server 
     * 
     * @param note It's a note objetc.     
     * 
     * @return Boolean variable
     */	
	public boolean createNote (Note note, Integer LayerId)
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "layer/" + LayerId.toString() + "/note/upload/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
			nvps.add(new BasicNameValuePair("title", note.getTitle()));
			nvps.add(new BasicNameValuePair("text", note.getBody()));
		    nvps.add(new BasicNameValuePair("latitude", note.getLatitude().toString()));
		    nvps.add(new BasicNameValuePair("longitude", note.getLongitude().toString() ));
		    nvps.add(new BasicNameValuePair("altitude", note.getAltitude().toString() ));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
		    
		    if (isLogged())
		    	httpclient.setCookieStore(cookieStore);
		    
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
	        return processMessage(str);
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
	}

	public boolean deleteNode (Integer nodeId, String LayerId)
	{
	
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			Log.e("DALE", mUrl + "layer/" + LayerId + "/node/" + nodeId + "/delete/?format=JSON");
			HttpGet httpGet = new HttpGet(mUrl + "layer/" + LayerId + "/node/" + nodeId + "/delete/?format=JSON"); 
			
			if (isLogged())
				httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpGet);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			Log.e(TAG, str);
			
			//maybe this crash!
			return processMessage(str);
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
	}

	public void setPageNote (Integer pageNote)
	{
		mpageNote = pageNote;
	}
	
	public int[] comparePhoto (Context context, Photo photo, Integer layerId)
	{
		
		try
		{			
			ClientHttpRequest myclient = new ClientHttpRequest( context.getResources().getString(R.string.machineServer) + "compareImages/social/compare/?format=JSON");
			
			myclient.setHeader( "Cookie" ,
								cookieStore.getCookies().get(0).getName() + "=" + 
							    cookieStore.getCookies().get(0).getValue());
			
			
			Log.e(TAG,cookieStore.getCookies().get(0).getName());
			Log.e(TAG,cookieStore.getCookies().get(0).getValue());
			
			InputStream serverInput;
			if (layerId == null)
				serverInput = myclient.post (
						new Object[] {
								"latitude", photo.getLatitude().toString(),
								"longitude", photo.getLongitude().toString(),
								"radius", "5",
								"photo", new File(photo.getUrl()) 
						}
				);   
			else
				serverInput = myclient.post (
						new Object[] {
								"latitude", photo.getLatitude().toString(),
								"longitude", photo.getLongitude().toString(),
								"radius", "5",
								"layer_id", Integer.toString(layerId),
								"photo", new File(photo.getUrl()) 
						}
				); 
			
			BufferedReader in = new BufferedReader(new InputStreamReader(serverInput));
			
			String inputLine;			
			String reponseString = "";
			
			while ((inputLine = in.readLine()) != null) {
				Log.e(TAG,"EPPP:" + inputLine);
				reponseString = reponseString + inputLine;
			}
			

	        Log.e(TAG, reponseString);
	        
	        return parsePhotosComparation(reponseString);

		}
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
	}
    
	public boolean uploadPhoto (Photo photo, Integer layerId)
	{
		
		try
		{			
			ClientHttpRequest myclient = new ClientHttpRequest( mUrl + "layer/" + layerId.toString() +"/photo/upload/?format=JSON");
			
			myclient.setHeader( "Cookie" ,
								cookieStore.getCookies().get(0).getName() + "=" + 
							    cookieStore.getCookies().get(0).getValue());
			
			
			Log.e(TAG,cookieStore.getCookies().get(0).getName());
			Log.e(TAG,cookieStore.getCookies().get(0).getValue());
			
			InputStream serverInput = myclient.post (
														new Object[] {
															"latitude", photo.getLatitude().toString(),
										                    "longitude", photo.getLongitude().toString(),
										                    "altitude", photo.getAltitude().toString(),
										                    "name", photo.getName(),
										                    "description" , photo.getDescription(),
										                    "photo", new File(photo.getUrl()) 
														}
													 );                   
			
			BufferedReader in = new BufferedReader(new InputStreamReader(serverInput));
			
			String inputLine;			
			String reponseString = "";
			
			while ((inputLine = in.readLine()) != null) {
				Log.e(TAG,"EPPP:" + inputLine);
				reponseString = reponseString + inputLine;
			}
			

	        Log.e(TAG, reponseString);
	        
	        return processMessage(reponseString);

		}
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
	}
	
	public boolean uploadAudio (Audio audio, Integer layerId)
	{
		
		try
		{

			ClientHttpRequest myclient = new ClientHttpRequest( mUrl + "layer/" + layerId.toString() + "/sound/upload/?format=JSON");
			
			myclient.setHeader( "Cookie" ,
								cookieStore.getCookies().get(0).getName() + "=" + 
							    cookieStore.getCookies().get(0).getValue());
			
			
			Log.e(TAG,cookieStore.getCookies().get(0).getName());
			Log.e(TAG,cookieStore.getCookies().get(0).getValue());
			
			InputStream serverInput = myclient.post (
														new Object[] {
															"latitude", 	audio.getLatitude().toString(),
										                    "longitude", 	audio.getLongitude().toString(),
										                    "altitude", 	audio.getAltitude().toString(),
										                    "name", 		audio.getName(),
										                    "description" , audio.getDescription(),
										                    "sound", 		new File(audio.getPath() ) 
														}
													 );                   
			
			BufferedReader in = new BufferedReader(new InputStreamReader(serverInput));
			
			String inputLine;			
			String reponseString = "";
			
			while ((inputLine = in.readLine()) != null) {
				Log.e(TAG,"EPPP:" + inputLine);
				reponseString = reponseString + inputLine;
			}
			

	        Log.e(TAG, reponseString);
	        
	        return processMessage(reponseString);

		}
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
	}
	
	public boolean setPrivacyNode (Integer nodeId, String privacyLevel, Boolean updateLocation)
	{
		Log.e("SetPrivacyNode","nodeId: " + nodeId.toString() + " privacy: " + privacyLevel);
		Log.e("SetPrivacyLocation: ", updateLocation.toString());
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			HttpPost httpost;
			
			httpost = new HttpPost(mUrl + "node/" + nodeId.toString() + "/privacy/change/?format=JSON"); 

			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
			
			if (updateLocation)
				nvps.add(new BasicNameValuePair("field", "position"));
			
			nvps.add(new BasicNameValuePair("perm", privacyLevel));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));		    		    
			httpclient.setCookieStore(cookieStore);

			HttpResponse response = httpclient.execute(httpost);
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);

	        return processMessage(str);
	        
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
		
	}
	
	public InputStream getStream (String url)
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + url);
			
			/* create cookie */
			if (isLogged()){
				List <NameValuePair> nvps = new ArrayList <NameValuePair>();			
				httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));		    		    
				httpclient.setCookieStore(cookieStore);
			}
			/* Send and receive the petition */
			HttpResponse response = httpclient.execute(httpost);
		    HttpEntity entity = response.getEntity(); 
		    
		    return entity.getContent();
		   	
		    
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
	}
	
	
	public Bitmap getBitmap (String url)
	{
		final int IO_BUFFER_SIZE = 4 * 1024;
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
	
			HttpPost httpost = new HttpPost(mUrl + url);
			
			Log.e("getBitMap:",mUrl + url);
			
			/* create cookie */
			if (isLogged()){
				List <NameValuePair> nvps = new ArrayList <NameValuePair>();			
				httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));	
				httpclient.setCookieStore(cookieStore);
			}
			/* Send and receive the petition */
			HttpResponse response = httpclient.execute(httpost);
		    HttpEntity entity = response.getEntity(); 
		    
		    InputStream in = entity.getContent();
		    
		    /* Convert response to bitmap */
            final ByteArrayOutputStream dataStream = new ByteArrayOutputStream();
            BufferedOutputStream out = new BufferedOutputStream(dataStream, IO_BUFFER_SIZE);
            BitmapUtils.copy(in, out);
            out.flush();
            
            final byte[] data = dataStream.toByteArray();
            Bitmap bitmap = BitmapFactory.decodeByteArray(data, 0, data.length);
	        
            return bitmap;
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
	}
	
	public ArrayList<GeoNode> getMyPhotos ()
	{
	
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			HttpPost httpost;
			
			httpost = new HttpPost(mUrl + "user/" + mUser.getId().toString() + "/photos/?format=JSON"); 

			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));		    		    
			httpclient.setCookieStore(cookieStore);

			HttpResponse response = httpclient.execute(httpost);
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);

	        return parseNodes(str);
	        
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
	}
	
	public void setPagePhoto (Integer pagePhoto)
	{
		mpagePhoto = pagePhoto;
	}
		
    private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        
        return sb.toString();

    }
	
	
    /*
     * Privacy
     * 
     * */
    
    public static String PRIVACY_PUBLIC 			= "Public";
    public static String PRIVACY_FRIENDS_OF_FRIENDS = "Friends of friends";
    public static String PRIVACY_FRIENDS 			= "Friends";
    public static String PRIVACY_PRIVATE			= "Private";
   
    
    public boolean setProfilePrivacy (String privacyLevel)
    {
    
		try
		{
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/privacy/change/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("perm", privacyLevel));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
	        return processMessage(str);
	                
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
    	
    }
    
    public boolean setLocationPrivacy (String privacyLevel)
    {
    
		try
		{
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/privacy/allow/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("field", "position"));
		    nvps.add(new BasicNameValuePair("role", privacyLevel));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
	        return processMessage(str);
	                
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
    	
    }
    
    public boolean setTags (Integer id, String tags)
    {
    	
		try
		{
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "node/" + id.toString() + "/tag/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("tags", tags));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
	        return processMessage(str);
	                
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
    	
    }
  
	private int[] parsePhotosComparation (String str)
	{
		
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.getInt("code") != 200)
				return null;
			
			if (json.has("photo_id") && json.has("layer_id")){
				int[] res = {json.getInt("layer_id"), json.getInt("photo_id")};
				return res;
			}else
				return null;
			
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}
		
		
	}
    
	private boolean processMessage (String str)
	{
		
        try {
        	JSONObject json;
			json = new JSONObject(str);
			lastCode = json.getInt("code");
			lastMsg  = json.getString("description");
			
			if (json.has("id"))
				lastId = json.getInt("id");
			else
				lastId = -1;
			
			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		
		return true;
		
	}
	
	public Boolean isLogged(){
		if (loged == false){
			return false;
		}
		else 
			return true;
	}
	
	public ArrayList<Category> parseCategories(Integer layerId){
		
		Integer id = null;
		String name = "";
		String description = "";
		
		ArrayList<Category> list = new ArrayList<Category>();
		
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();						
			HttpGet httpGet = new HttpGet(mUrl + "layer/" + layerId.toString() + "/categories/?format=JSON");	
			
			if (isLogged())
				httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpGet);
			HttpEntity entity = response.getEntity(); 
			String str = convertStreamToString(entity.getContent());
        
			JSONObject json;
			
			try{	
				json = new JSONObject(str);
				lastCode = json.getInt("code");
								
				if (lastCode != 200)
					return null;
				
				JSONArray array;
				array = json.getJSONArray("results");
				
				for (int i = 0; i < array.length(); i++) 
				{
					JSONObject obj = array.getJSONObject(i);
					
					if (array.length() == 1){
						id = 0;
						name = "No categories";
						description = "this layer has not categories";
						Category one_category = new Category(id, name, description);
						list.add(one_category);
						break;
					}
					if (obj.has("id")){
						id = obj.getInt("id");
						if (id == 0)
							continue;
					}
					if (obj.has("name"))
						name = obj.getString("name");
					if (obj.has("description"))
						description = obj.getString("description");
					
					Category category = new Category(id, name, description);
					list.add(category);
					
					
				}
				/*
				if (list.isEmpty()){
					id = 0;
					name = "all";
					description = "";
					Category category = new Category(id, name, description);
					list.add(category);
				}*/

			} catch (JSONException e) {	
				lastCode = 500;
				lastMsg = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, e.toString());
				return null;
			}	
			
		
		} catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}		
		
		return list;
	}
	
	private ArrayList<GeoNode>  parseNodes (String str)
	{
		 
        JSONObject json;
        
        ArrayList<GeoNode> list = new ArrayList<GeoNode>();
        
        try {
			json = new JSONObject(str);
			lastCode = json.getInt("code");
							
			if (lastCode != 200)
				return null;
			
			JSONArray array;
			
			array = json.getJSONArray("results");

			Log.e("parseNodes_new: ",String.valueOf(array.length()));
			Log.e("PARSE", array.toString());
			
			for (int i = 0; i < array.length(); i++) 
			{ 

				Log.e("PARSE", array.getJSONObject(i).toString());
				GeoNode node = parseNode (array.getJSONObject(i));
							
				list.add(node);
				
			}
			
			return list;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}
	}
	
	private GeoNode  parseNode (JSONObject obj)
	{
		String type = "";
		String since, position_since, username;
		Double latitude, longitude, altitude, radius, distance;
		Integer user_id = 0;
		Integer id = 0;
		latitude = longitude = altitude = radius = distance = -1.0;
		since = position_since = "";
		username = "";
		
		GeoNode node = null;

		try
		{
			if (obj.has("type")){
				type = obj.getString("type");
			}else{
				Log.e("Parse node error: ", "no type in node");
				return null;
			}		

			if (obj.has("id"))
				id = obj.getInt("id");
			
			if (obj.has("since"))
				since = obj.getString("since");	
								
			if (obj.has("position"))
			{
				JSONObject jsonPosition = obj.getJSONObject("position");
				if (jsonPosition.has("latitude"))
					latitude = jsonPosition.getDouble("latitude");
				if (jsonPosition.has("longitude"))
					longitude = jsonPosition.getDouble("longitude");
				if (jsonPosition.has("altitude"))
					altitude = jsonPosition.getDouble("altitude");
				if (jsonPosition.has("since"))
					position_since = jsonPosition.getString("since");
				
				if (obj.has("radius"))
					radius = jsonPosition.getDouble("radius");									
			}
			
			if (obj.has("uploader"))
			{
				JSONObject jsonPosition = obj.getJSONObject("uploader");
				if (jsonPosition.has("username"))
					username = jsonPosition.getString("username");
				if (jsonPosition.has("id"))
					user_id = jsonPosition.getInt("id");
									
			}
			
			// parse for person type
			if (type.equalsIgnoreCase("person")){
				User user = parseUser(obj);
				if (user == null){
					Log.e(TAG, "Parse user returns NULL");
					return null;
				}
				Log.i("ciudad", user.getCountry());
				node = (GeoNode) user;
			}
	
			if (type.equalsIgnoreCase("photo")){
				Photo photo = parsePhoto(obj);
				if (photo == null){
					Log.e(TAG, "Parse photo returns NULL");
					return null;
				}
				node = (GeoNode) photo;
			}
			if (type.equalsIgnoreCase("note")){
				Note note = parseNote(obj);
				if (note == null){
					Log.e(TAG, "Parse note returns NULL");
					return null;
				}
				node = (GeoNode) note;
			}
			if (type.equalsIgnoreCase("sound")){
				Audio audio = parseAudio(obj);
				if (audio == null){
					Log.e(TAG, "Parse sound returns NULL");
					return null;
				}
				node = (GeoNode) audio;
			}
			if (type.equalsIgnoreCase("video")){
				Video video = parseVideo(obj);
				if (video == null){
					Log.e(TAG, "Parse sound returns NULL");
					return null;
				}
				node = (GeoNode) video;
			}
			if (node == null){
				Log.e(TAG,"NODE WAS NULL");
				return null;
			}
			
			node.setAltitude(altitude.floatValue());
			node.setLatitude(latitude);
			node.setLongitude(longitude);
			node.setRadius(radius);
			node.setSince(since);
			node.setId(id);
			node.setPosition_since(position_since);
			node.setUsername(username);
			node.setUser_id(user_id);
			
			
			return node;
			
		} catch (JSONException e) {	
			
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}
	}
	
	private Audio parseAudio (JSONObject obj){
		String description = null;
		String name = null;
		String url = null;
		Integer id = null;
		
		try {

			if (obj.has("name"))
				name = obj.getString("name");
			if (obj.has("description"))
				description = obj.getString("description");	
			if (obj.has("id")){
				id = obj.getInt("id");
			
				
			Integer layerId = (Integer) getLayerId(id).get(0);
			Log.e("LAYER", layerId.toString());
			url = "layer/" + layerId.toString() + "/node/" + id.toString() + "/sound_file/";
			}
			return new Audio (0,name,description,url,"",0.0,0.0,0.0,0.0,null, null, 0.0);
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}	 

	}
	
	private Video parseVideo (JSONObject obj){
		String description = null;
		String name = null;
		String info_url, video_url, video_thumb_url;
		info_url = video_url = video_thumb_url = "";
		
		try {

			if (obj.has("name"))
				name = obj.getString("name");
			if (obj.has("description"))
				description = obj.getString("description");	
			if (obj.has("external_info")){
				JSONObject jsonInfo = obj.getJSONObject("external_info");
				info_url = jsonInfo.getString("info_url");
				video_url= jsonInfo.getString("video_url");
				video_thumb_url = jsonInfo.getString("video_thumb_url");
			}
			return new Video (0,name,description,info_url, video_url, video_thumb_url,"",0.0,0.0,0.0,0.0,null, null, 0.0);
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}	 

	}
	
	private Photo parsePhoto (JSONObject obj){
		String description = null;
		String name = null;
		String url = null;	
		Integer id = null;
				
		try {
			
			if (obj.has("id"))
				id = obj.getInt("id");
			else
				id = 0;
			
			if (obj.has("name"))
				name = obj.getString("name");
			if (obj.has("description"))
				description = obj.getString("description");	
		
			/* If id=0 then the geonode is external, 
			 * and the image is saved in photo_url field */
			
			ExtenalInfo extInfo = null;
			url = null;
			
			if (id==0)
			{
				extInfo = new ExtenalInfo();
				
				if (obj.has("external_info"))
				{
					JSONObject objPosition = obj.getJSONObject("external_info");
					
					if (objPosition.has("info_url"))
						extInfo.setUrlInfo(objPosition.getString("info_url"));
					
					if (objPosition.has("photo_thumb"))
					{
						extInfo.setPhotoThumbUrl (objPosition.getString("photo_thumb"));
						url = extInfo.getPhotoThumbUrl();
					}
					
					if (objPosition.has("photo_url"))
						extInfo.setPhotoNormalUrl (objPosition.getString("photo_url"));
				}
				
			}
			
			else{
				Log.e("LAYER", id.toString());
				Integer layerId = (Integer) getLayerId(id).get(0);
				url = "layer/" + layerId.toString() + "/node/" + id.toString() + "/image/";
			}
			Photo myPhoto = new Photo (id,0.0,0.0,0.0,0.0,name,description,url,null, null, null, 0.0);
			myPhoto.setExternalInfo (extInfo);
			
			return myPhoto;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}
	}
	
	private Note parseNote (JSONObject obj){
		String title, text;
		title = text  = "";
		
		try {			
			
			if (obj.has("title"))
				title = obj.getString("title");
			
			if (obj.has("text"))
				text = obj.getString("text");
				
			return new Note (0,title,text,0.0,0.0, 0.0, 0.0, null, null, 0.0);
		
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG+"-parseNote", e.toString());
			return null;
		}
	}
	
	private User parseUser (JSONObject obj){
		
		String username = "", name = "", lastName ="", status="", status_since="";
		String email ="";
		String birthDay="";
		Integer avatarId = null;
		String avatarUrl = "";
		String country = "";
		Integer postCode = 0;
		try{
			if (obj.has("first_name"))
				name = obj.getString("first_name");	
			
			if (obj.has("last_name"))
				lastName = obj.getString("last_name");

			if (obj.has("username"))
				username = obj.getString("username");

			if (obj.has("status")){
				JSONObject jsonStatus = obj.getJSONObject("status");
				status = jsonStatus.getString("message");
				status_since = jsonStatus.getString("since");
			}

			if (obj.has("email"))
				email = obj.getString("email");
		
			if (obj.has("birthday"))
				birthDay = obj.getString("birthday");

			if (obj.has("avatar"))
			{
				JSONObject jsonPosition = obj.getJSONObject("avatar");
			
				avatarId = jsonPosition.getInt("photo_id");
				avatarUrl = jsonPosition.getString("photo_url");
			}

			if (obj.has("position"))
			{
				JSONObject jsonPosition = obj.getJSONObject("position");
				Log.i("POSITION", jsonPosition.toString());
				if (jsonPosition.has("country"))
					country = jsonPosition.getString("country");
				
				if (jsonPosition.has("post_code"))
					postCode = jsonPosition.getInt("post_code");
				
			}
			
			return new User(null,name,lastName,username,email,status,0.0,0.0,0.0,0.0, 
					null,country,postCode,birthDay,null, avatarId, avatarUrl, status_since, null);
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}
	}
	
	public ArrayList getLayerId (Integer nodeId){
		Integer id = 0;
		String name = "";
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpGet httpGet = new HttpGet(mUrl + "node/" + nodeId.toString() + "/layers/?format=JSON");						
			
			if (isLogged())
				httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpGet);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);

	        
	        JSONObject json;
	        
	        ArrayList list = new ArrayList();
	        
	        try {
				json = new JSONObject(str);
				lastCode = json.getInt("code");
								
				if (lastCode != 200)
					return null;
				
				JSONArray array;
				
				array = json.getJSONArray("results");
				
				for (int i = 0; i < array.length(); i++) 
				{ 
					JSONObject layer = array.getJSONObject(i);
					
					if (layer.has("id"))
						id = layer.getInt("id");
					list.add(id);
				}
				return list;
	        } catch (JSONException e) {	
				lastCode = 500;
				lastMsg = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, e.toString());
				return null;
			}

            
		}catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
	}
	
	public ArrayList<GenericLayer> getLayerList(){
		
		String layer_type, name, description, since;
		Integer id = 0;
		Double altitude, distance;
		String position_since;
		
		layer_type = name = description = since = position_since = "";
		altitude = distance = 0.0;
		
		Bitmap icon = null;
		
		try
		{
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpGet httpGet = null;
			
			if (isLogged()){
				httpclient.setCookieStore(cookieStore);
				httpGet = new HttpGet(mUrl + "layer/list/?format=JSON&user_layer=true");
			} else {
				httpGet = new HttpGet(mUrl + "layer/list/?format=JSON");
			}
			
			HttpResponse response = httpclient.execute(httpGet);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);

	        
	        JSONObject json;
	        
	        ArrayList<GenericLayer> list = new ArrayList<GenericLayer>();
	        
	        try {
				json = new JSONObject(str);
				lastCode = json.getInt("code");
								
				if (lastCode != 200)
					return null;
				
				JSONArray array;
				
				array = json.getJSONArray("results");

				Log.e("parseNodes_new: ",String.valueOf(array.length()));
				Log.e("PARSE", array.toString());
				
				for (int i = 0; i < array.length(); i++) 
				{ 
					Log.e("PARSE", array.getJSONObject(i).toString());
					JSONObject layer = array.getJSONObject(i);
					
					if (layer.has("layer_type"))
						layer_type = layer.getString("layer_type");
					if (layer.has("name"))
						name = layer.getString("name");
					if (layer.has("since"))
						since = layer.getString("since");
					if (layer.has("id"))
						id = layer.getInt("id");
					if (layer.has("description"))
						description = layer.getString("description");
					
					JSONObject position = layer.getJSONObject("position");
					
					if (position.has("since"))
						position_since = position.getString("since");
					if (position.has("distance"))
						distance = position.getDouble("distance");
					if (position.has("altitude"))
						altitude = position.getDouble("altitude");
					
					icon = LibreGeoSocial.getInstance().getBitmap("layer/" + id.toString() + "/icon/" );
					
					GenericLayer mLayer =  new GenericLayer(id, layer_type, name, description,  since,
							null, null, altitude, distance, position_since );

					list.add(mLayer);
					
				}
				
				return list;
				
			} catch (JSONException e) {	
				lastCode = 500;
				lastMsg = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, e.toString());
				return null;
			}

            
		}catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
	}
	
	
	
	public static void allowAllSSL(){
		
		javax.net.ssl.HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier(){

			public boolean verify(String hostname, SSLSession session) {
				// TODO Auto-generated method stub
				return true;
			}
			
		});
		
		javax.net.ssl.SSLContext context = null;
		
		if(trustManagers==null){
			trustManagers = new javax.net.ssl.TrustManager[]{new _FakeX509TrustManager()};
		}
		
		try{
			context = javax.net.ssl.SSLContext.getInstance("TLS");
			context.init(null, trustManagers, new SecureRandom());
			
		}catch(NoSuchAlgorithmException e){
			Log.e("allowAllSSL", e.toString());
		}catch(KeyManagementException e){
			Log.e("allowAllSSL", e.toString());
		}
		
		javax.net.ssl.HttpsURLConnection.setDefaultSSLSocketFactory(context.getSocketFactory());
		
	}
	
	public static class _FakeX509TrustManager implements javax.net.ssl.X509TrustManager{
		
		private static final X509Certificate[] _AcceptedIssuers = new X509Certificate[]{};
		
		public void checkClientTrusted(X509Certificate[] chain, String authType)
				throws CertificateException {
			// TODO Auto-generated method stub
			
		}

		public void checkServerTrusted(X509Certificate[] chain, String authType)
				throws CertificateException {
			// TODO Auto-generated method stub
			
		}
		
		public boolean isClientTrusted(X509Certificate[] chain){
			return true;
		}
		
		public boolean isServerTrusted(X509Certificate[] chain){
			return true;
		}
		
		public X509Certificate[] getAcceptedIssuers() {
			// TODO Auto-generated method stub
			return _AcceptedIssuers;
		}
		
	}
	
}
