from django.db import models
from datetime import datetime
from django.contrib.gis.db import models
from social.core.models import *
from social.core.models import Social_node

from django.contrib.auth.models import User

class Organizador(models.Model):          
    usuario = models.ForeignKey(Person,null=False)
    partidas = models.ManyToManyField("Partida",blank=True,null=True)

class Partida(models.Model):
    titulo = models.CharField(max_length=200,null=False)
    lugar = models.CharField(max_length=200,null=False)
    fecha = models.DateField(default=datetime.now(),null=False)
    duracion = models.IntegerField(default=0,null=False)
    espias = models.IntegerField(default=0,null=False)
    escolta = models.IntegerField(default=0,null=False)
    crecimiento = models.CharField(max_length=10,null=False)
    descripcion = models.TextField(null=False)
    cerrado = models.BooleanField(default=False,null=False)

class Color(models.Model):
    nombre = models.CharField(max_length=30,null=False)
    partida = models.ForeignKey("Partida",null=False)

class Equipo(models.Model):
    grupo = models.ForeignKey(Group,null=False)
    partida = models.ForeignKey("Partida",null=False)
    color = models.CharField(max_length=30,null=False)
    espias = models.IntegerField(default=0,null=False)
    escolta = models.IntegerField(default=0,null=False)
    version = models.IntegerField(default=0,null=False)

class Marcador(models.Model):
    partida = models.ForeignKey("Partida",null=False)
    equipo = models.ForeignKey("Equipo",null=False)
    puntuacion = models.IntegerField(default=0,null=False)
    num_territorios = models.IntegerField(default=0,null=False)
    
class Territorio(Social_node):
    nombre = models.CharField(max_length=200,null=False)
    partida = models.ForeignKey("Partida",null=False)
    equipo = models.ForeignKey("Equipo",blank=True,null=True)
    version = models.IntegerField(default=0,null=False)

class Tropa(models.Model):
    territorio = models.ForeignKey("Territorio",null=False)
    tipo = models.CharField(max_length=50,null=False)
    cantidad = models.IntegerField(default=0,null=False)

class Conquista(models.Model):
    atacante = models.ForeignKey("Equipo",null=False,related_name="equipo_atacante_conquista")
    defensor = models.ForeignKey("Equipo",null=False,related_name="equipo_defensor_conquista")
    territorio = models.ForeignKey("Territorio",null=False)
    victoria = models.BooleanField(default=False,null=False)

class Duelo(models.Model):
    atacante = models.ForeignKey("Equipo",null=False,related_name="equipo_atacante_duelo")
    defensor = models.ForeignKey("Equipo",null=False,related_name="equipo_defensor_duelo")
    victoria = models.BooleanField(default=False,null=False)
    
class Mensaje(models.Model):
    partida = models.ForeignKey("Partida",null=False)
    desde_organizador = models.ForeignKey("Organizador",blank=True,null=True,related_name="organizador_envia")
    desde_equipo = models.ForeignKey("Equipo",blank=True,null=True,related_name="equipo_envia")
    a_organizador = models.ForeignKey("Organizador",blank=True,null=True,related_name="organizador_recibe")
    a_equipo = models.ManyToManyField("Equipo",blank=True,null=True,related_name="equipo_recibe")
    texto = models.TextField(null=False)
    date = models.DateTimeField(default=datetime.now)
    
