# Create your views here.

from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.contrib.gis.geos import Point
from django.contrib.auth import authenticate, login
from django.contrib.auth import logout

from social.core import api as api_lgs
from social.rest.forms import LoginForm

from apps.conquistadores.models import *
from apps.conquistadores.core import lib_organizador
from apps.conquistadores.core import lib_partida
from apps.conquistadores.core import lib_territorio
from apps.conquistadores.core import lib_equipo
from apps.conquistadores.core import lib_tropa
from apps.conquistadores.core import lib_marcador
from apps.conquistadores.core import lib_mensaje
from apps.conquistadores.core import lib_color
from apps.conquistadores.core import lib_conquista
from apps.conquistadores.core import lib_duelo

ORGANIZADOR_NOMBRE = 'javier'
ORGANIZADOR_PWD = 'django'

def chequear_formato(request):
    if "format" in request.GET:
        formato = request.GET["format"].lower().strip()
    else:
        if "format" in request.POST:
            formato = request.POST["format"].lower().strip()
        else:
            formato = 'html'

    if formato == 'json':
      return True, 'json'
    if formato == 'html':
      return True, 'html'
    #if formato == 'xml':
      #return True, 'xml'
    return False, "Formato no valido. Solo .json y .html"

def login_usuario(request):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    if not request.user.is_authenticated():        
        if request.method == 'GET':
            organizadores = lib_organizador.listar_completo()
            if len(organizadores) == 0:
                organizador = {'username': ORGANIZADOR_NOMBRE, 'password': ORGANIZADOR_PWD}
                correcto, resultado = api_lgs.user.create_or_modify(organizador, modify=True)
                if correcto:
                    usuario = Person.objects.get(id=resultado)
                    lib_organizador.crear(usuario)
                    return render_to_response('login.' + formato, {'codigo': 200, 'descripcion': 'ok'})
                else:
                    msgError = resultado
                    return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': msgError})
            return render_to_response('login.html')        
        elif request.method == 'POST':
            loginform = LoginForm(request.POST)
            id = loginform.login(request)
            if id:
                url = "/conquistadores/inicio/"
                return HttpResponseRedirect(url)
            else:
                return render_to_response('login.' + formato, {'codigo': 200, 'descripcion': 'ok', 'falloAutent': 1})
    else:
        url = "/conquistadores/inicio/"
        return HttpResponseRedirect(url)
        
def logout_usuario(request):
    if request.user.is_authenticated():
        logout(request)
    return HttpResponseRedirect("/conquistadores/usuario/login/")

def usr_partida_listar(request):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    if request.method != 'GET':
      return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        usuario_id = request.session["_auth_user_id"]
        correcto, resultado = lib_partida.listar_organizador(usuario_id)
        if correcto:
            partidas = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        if len(partidas) == 0:
            msginfo = "No se encuentran partidas creadas para este organizador"
            return render_to_response('inicio.' + formato, {'codigo': 200, 'descripcion': 'No hay partidas', 'admin': admin, 'msginfo': msginfo})
        else:
            return render_to_response('inicio.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partidas': partidas})
    else:
        return render_to_response('inicio.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin})  

    
# FUNCIONES PARTIDA
def partida_crear(request):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        if request.method == 'GET':
            return render_to_response('partida_crear.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin})
        elif request.method == 'POST':
            titulo = request.POST["titulo"].strip()
            lugar = request.POST["lugar"].strip()
            fecha = request.POST["fecha"].strip()
            duracion = request.POST["duracion"].strip()
            espias = request.POST["espias"].strip()
            escolta = request.POST["escolta"].strip()
            crecimiento = request.POST["crecimiento"].strip()
            descripcion = request.POST["descripcion"].strip()
            usuario_id = request.session["_auth_user_id"]
            partida = lib_partida.crear(titulo, lugar, fecha, duracion, espias, escolta, crecimiento, descripcion, usuario_id)
            lib_color.crear(partida);
            url = "/conquistadores/partida/"+str(partida.id)+"/info/"
            return HttpResponseRedirect(url)    
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def partida_modificar(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

        if request.method == 'GET':
            return render_to_response('partida_modificar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida})
        elif request.method == 'POST':
            partida_ant = resultado
            titulo = request.POST["titulo"].strip()
            lugar = request.POST["lugar"].strip()
            fecha = request.POST["fecha"].strip()
            duracion = request.POST["duracion"].strip()
            espias = request.POST["espias"].strip()
            escolta = request.POST["escolta"].strip()
            crecimiento = request.POST["crecimiento"].strip()
            descripcion = request.POST["descripcion"].strip()
            partida = lib_partida.modificar(titulo, lugar, fecha, duracion, espias, escolta, crecimiento, descripcion, partida_ant)
            equipos = lib_equipo.listar_completo(partida)
            for equipo in equipos:
                lib_equipo.modificar_por_partida(partida.espias, partida.escolta, equipo)    
            url = "/conquistadores/partida/"+str(partida.id)+"/info/"
            return HttpResponseRedirect(url)
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def partida_eliminar(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
    
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
            id_usuario = request.session["_auth_user_id"]
            lib_partida.eliminar(partida, id_usuario)
            url = "/conquistadores/partida/listar"
            return HttpResponseRedirect(url)
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def partida_info(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
      return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
        admin = lib_organizador.comprobar_tipo_usuario(request)
        return render_to_response('partida_info.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        
def partida_cambiarEstado(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
      return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
            partida = lib_partida.cambiar_estado(partida)
            return HttpResponseRedirect('/conquistadores/partida/listar')
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})
    
def partida_listar(request):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
       
    if request.method != 'GET':
      return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    partidas = lib_partida.listar_completo()
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if len(partidas) == 0:
        return render_to_response('partida_listar.' + formato, {'codigo': 200, 'descripcion': 'No hay partidas', 'admin': admin})
    else:
        return render_to_response('partida_listar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partidas': partidas})

def partida_seguimiento(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})    

    admin = lib_organizador.comprobar_tipo_usuario(request)
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
        territorios = lib_territorio.listar_completo(partida)
        marcadores = lib_marcador.listar_completo(partida)
        mensajes = lib_mensaje.listar_completo(partida)
        tropas = lib_tropa.listar_completo(partida)
        return render_to_response('partida_seguimiento.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'territorios': territorios, 'tropas':tropas, 'marcadores': marcadores, 'mensajes': mensajes})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def partida_estadisticas(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
      return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})        

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
        marcadores = lib_marcador.listar_completo(partida)
        admin = lib_organizador.comprobar_tipo_usuario(request)
        return render_to_response('partida_estadisticas.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'miEquipoId':int(equipo_id), 'marcadores': marcadores})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def partida_recurso_listar(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:            
        partida = resultado            
        admin = lib_organizador.comprobar_tipo_usuario(request)
        equipos = lib_equipo.listar_completo(partida)
        territorios = lib_territorio.listar_completo(partida)
        if len(equipos) == 0:
            return render_to_response('partida_recurso_listar.' + formato, {'codigo': 200, 'descripcion': 'no equipos', 'admin': admin})
        elif len(territorios) == 0:
            return render_to_response('partida_recurso_listar.' + formato, {'codigo': 200, 'descripcion': 'no territorios', 'admin': admin})
        else:
            return render_to_response('partida_recurso_listar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'equipos':equipos, 'territorios':territorios})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
     
#FUNCIONES EQUIPO
def equipo_crear(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        if request.method == 'GET':
            colores = lib_color.listar_disponibles(partida)
            return render_to_response('equipo_crear.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'colores': colores})
        
        elif request.method == 'POST':
            nombre = request.POST["nombre"].strip()
            color_id = request.POST["colores"].strip()
            correcto, resultado = lib_color.obtener(partida, color_id)
            if correcto:
                color = resultado
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})   

            correcto, resultado = lib_equipo.crear(nombre, color, partida)
            if correcto:
                equipo = resultado
                url = "/conquistadores/partida/"+str(partida.id)+"/equipo/"+str(equipo.id)+"/info/"
                return HttpResponseRedirect(url)
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def equipo_modificar(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})    
        correcto, resultado = lib_equipo.obtener(partida, equipo_id)
        if correcto:
            if request.method == 'GET':
                equipo = resultado
                return render_to_response('equipo_modificar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'equipo': equipo})
        
            elif request.method == 'POST':
                equipo_ant = resultado
                nombre = request.POST["nombre"].strip()
                correcto, resultado = lib_equipo.modificar(nombre, equipo_ant)
                if correcto:
                    equipo = resultado
                    url = "/conquistadores/partida/"+str(partida.id)+"/equipo/"+str(equipo.id)+"/info/"
                    return HttpResponseRedirect(url)
                else:
                    msgError = resultado
                    return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def equipo_eliminar(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

        correcto, resultado = lib_equipo.obtener(partida, equipo_id)
        if correcto:
            equipo = resultado
            id_usuario = request.session["_auth_user_id"]
            lib_equipo.eliminar(equipo, id_usuario)
            url = "/conquistadores/partida/"+str(partida.id)+"/equipo/listar/"
            return HttpResponseRedirect(url)
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def equipo_listar(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:            
        partida = resultado
        equipos = lib_equipo.listar_completo(partida)
        admin = lib_organizador.comprobar_tipo_usuario(request)
        if len(equipos) == 0:
            return render_to_response('equipo_listar.' + formato, {'codigo': 200, 'descripcion': 'No hay eqipos', 'admin': admin, 'partida': partida})
        else:
            return render_to_response('equipo_listar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'equipos': equipos})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def equipo_info(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})        

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})    

    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

    if formato == 'json':
        return render_to_response('equipo_info.' + formato, {'codigo': 200, 'descripcion': 'ok', 'equipo': equipo})
    
    territorios = lib_territorio.obtener_por_equipo(equipo)
    correcto, resultado = lib_marcador.obtener(equipo)
    if correcto:
        marcador = resultado
        admin = lib_organizador.comprobar_tipo_usuario(request)
        return render_to_response('equipo_info.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'equipo': equipo, 'territorios': territorios, 'marcador':marcador})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def equipo_duelo(request, partida_id, equipo_atac_id, equipo_def_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_equipo.obtener(partida, equipo_atac_id)
    if correcto:
        equipo_atacante = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_equipo.obtener(partida, equipo_def_id)
    if correcto:
        equipo_defensor = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

    if request.method == 'GET':     
        return render_to_response('equipo_duelo.'+formato, {'codigo': 200, 'descripcion': 'ok', 'equipoAtacante':equipo_atacante, 'equipoDefensor':equipo_defensor})
    
    elif request.method == 'POST':        
        equipo_ganador_id = int(request.POST["ganadorId"].strip())
        defensor_version = int(request.POST["equipoDefensorVersion"].strip())
        atacante_version = int(request.POST["equipoAtacanteVersion"].strip())
        atac_escolta_final = int(request.POST["atacanteEscoltaFinal"].strip())
        def_escolta_final = int(request.POST["defensorEscoltaFinal"].strip())
        puntuacion = int(request.POST["puntuacion"].strip())

        correcto, resultado = lib_equipo.duelo(partida, equipo_ganador_id, defensor_version, atacante_version, atac_escolta_final,
                                               def_escolta_final, equipo_atacante, equipo_defensor, puntuacion)
        if correcto:
            return render_to_response('respuesta.'+formato, {'codigo': 200, 'descripcion': resultado})
        else:
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': resultado}) 
    
# FUNCIONES TERRITORIO
def territorio_crear(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        if request.method == 'GET':
            return render_to_response('territorio_crear.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida})
        
        elif request.method == 'POST':
            nombre = request.POST["nombre"].strip()
            latitud = float(request.POST["latitud"].strip().replace(",","."))
            longitud = float(request.POST["longitud"].strip().replace(",","."))
            posicion = Point(longitud, latitud, srid=4326)
            tipo = request.POST["tipo"].strip()
            cantidad = request.POST["cantidad"].strip()
            territorio = lib_territorio.crear(nombre, posicion, partida)
            tropa = lib_tropa.crear(tipo, cantidad, territorio)
            
            url = "/conquistadores/partida/"+str(partida.id)+"/territorio/"+str(territorio.id)+"/info/"
            return HttpResponseRedirect(url)    
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def territorio_modificar(request, partida_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_territorio.obtener(partida, territorio_id)
        if correcto:
            territorio = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        tropas = lib_tropa.obtener_por_territorio(territorio)
        if request.method == 'GET':                  
            return render_to_response('territorio_modificar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida,'territorio': territorio, 'tropas': tropas})
        elif request.method == 'POST':
            territorio_ant = resultado
            nombre = request.POST["nombre"].strip()
            latitud = float(request.POST["latitud"].strip().replace(",","."))
            longitud = float(request.POST["longitud"].strip().replace(",","."))
            posicion = Point(longitud, latitud, srid=4326)
            tipo = request.POST["tipo"].strip()
            cantidad = request.POST["cantidad"].strip()
            territorio = lib_territorio.modificar(nombre, posicion, territorio_ant)
            for tropa_ant in tropas:
                tropa = lib_tropa.modificar(tipo, cantidad, tropa_ant)
            url = "/conquistadores/partida/"+str(partida.id)+"/territorio/"+str(territorio.id)+"/info/"
            return HttpResponseRedirect(url)
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def territorio_eliminar(request, partida_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
    
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_territorio.obtener(partida, territorio_id)
        if correcto:
            territorio = resultado
            id_usuario = request.session["_auth_user_id"] 
            lib_territorio.eliminar(territorio, id_usuario)
            url = "/conquistadores/partida/"+str(partida.id)+"/territorio/listar/"
            return HttpResponseRedirect(url)
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 500, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def territorio_info(request, partida_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_territorio.obtener(partida, territorio_id)
    if correcto:
        territorio = resultado
        tropas = lib_tropa.obtener_por_territorio(territorio)
        admin = lib_organizador.comprobar_tipo_usuario(request)
        return render_to_response('territorio_info.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'equipo': territorio.equipo, 'territorio': territorio, 'tropas': tropas})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def territorio_listar(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
        territorios = lib_territorio.listar_completo(partida)
        admin = lib_organizador.comprobar_tipo_usuario(request)
        if len(territorios) == 0:
            return render_to_response('territorio_listar.' + formato, {'codigo': 200, 'descripcion': 'No hay territorios', 'admin': admin, 'partida': partida})
        else:
            return render_to_response('territorio_listar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'territorios': territorios})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

#FUNCIONES TROPA
def tropa_crear(request, partida_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_territorio.obtener(partida, territorio_id)
        if correcto:
            territorio = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        tropas = lib_tropa.listar_completo_territorio(territorio)
        if len(tropas) < 1:
            if request.method == 'GET':
                return render_to_response('tropa_crear.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'territorio': territorio})
    
            elif request.method == 'POST':
                tipo = request.POST["tipo"].strip()
                cantidad = request.POST["cantidad"].strip()
                tropa = lib_tropa.crear(tipo, cantidad, territorio)
                url = "/conquistadores/partida/"+str(partida.id)+"/territorio/"+str(territorio.id)+"/tropa/"+str(tropa.id)+"/info/"
                return HttpResponseRedirect(url)            
        else:
            msgError = "Ya existen tropas en este territorio."
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def tropa_modificar(request, partida_id, territorio_id, tropa_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_territorio.obtener(partida, territorio_id)
        if correcto:
            territorio = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_tropa.obtener(territorio, tropa_id)
        if correcto:
            if request.method == 'GET':
                tropa = resultado
                return render_to_response('tropa_modificar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'territorio': territorio, 'tropa': tropa, 'long_tropas': 1})
    
            elif request.method == 'POST':
                tropa_ant = resultado
                tipo = request.POST["tipo"].strip()
                cantidad = request.POST["cantidad"].strip()
                tropa = lib_tropa.modificar(tipo, cantidad, tropa_ant)
                url = "/conquistadores/partida/"+str(partida.id)+"/territorio/"+str(territorio.id)+"/tropa/"+str(tropa.id)+"/info/"
                return HttpResponseRedirect(url)
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def tropa_eliminar(request, partida_id, territorio_id, tropa_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_territorio.obtener(partida, territorio_id)
        if correcto:
            territorio = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_tropa.obtener(territorio, tropa_id)
        if correcto:
            tropa = resultado
            lib_tropa.eliminar(tropa)
            url = "/conquistadores/partida/"+str(partida.id)+"/territorio/"+str(territorio.id)+"/tropa/listar/"
            return HttpResponseRedirect(url)
        else:
            msgError = "No pudo eliminarse la tropa."
            return render_to_response('fallo.'+formato, {'codigo': 500, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

def tropa_info(request, partida_id, territorio_id, tropa_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_territorio.obtener(partida, territorio_id)
    if correcto:
        territorio = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_tropa.obtener(territorio, tropa_id)
    if correcto:
        tropa = resultado
        admin = lib_organizador.comprobar_tipo_usuario(request)
        return render_to_response('tropa_info.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'territorio': territorio, 'tropa':tropa, 'long_tropas': 1})   
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def tropa_listar(request, partida_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_territorio.obtener(partida, territorio_id)
    if correcto:
        territorio = resultado
        tropas = lib_tropa.obtener_por_territorio(territorio)
        admin = lib_organizador.comprobar_tipo_usuario(request)
        if len(tropas) == 0:
            return render_to_response('tropa_listar.' + formato, {'codigo': 200, 'descripcion': 'No hay tropas', 'admin': admin, 'partida': partida, 'territorio': territorio, 'long_tropas': 0})
        else:
            return render_to_response('tropa_listar.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'territorio': territorio, 'tropas': tropas, 'long_tropas': 1})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

#FUNCIONES DE UN EQUIPO SOBRE TERRITORIO
def equipo_territorio_conquistar(request, partida_id, equipo_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

    if request.method == 'GET':     
        correcto, resultado = lib_territorio.obtener(partida, territorio_id)
        if correcto:
            territorio = resultado
            tropaRival = lib_tropa.obtener_por_territorio(territorio)
            return render_to_response('equipo_territorio_conquistar.'+formato, {'codigo': 200, 'descripcion': 'ok', 'equipo':equipo, 'tropaRival':tropaRival})
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    elif request.method == 'POST':            
        correcto, resultado = lib_marcador.obtener(equipo)
        if correcto:
            marcador_atacante = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

        equipo_defensor_id = request.POST["equipoDefensorId"].strip()                        
        puntuacion = int(request.POST["puntuacion"].strip())
        #El equipo conquista un territorio sin duenyo
        if (equipo_defensor_id == '-1'):
            correcto, msg = lib_territorio.conquistar_vacio(partida, territorio_id, marcador_atacante, puntuacion)  
            if correcto:
                return render_to_response('respuesta.'+formato, {'codigo': 200, 'descripcion': msg})
            else:
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msg})

        #El equipo conquista un territorio rival
        else:                            
            equipo_ganador_id = int(request.POST["ganadorId"].strip())
            version_atacante = request.POST["equipoAtacanteVersion"].strip()  
            equipo_defensor_id = request.POST["equipoDefensorId"].strip()                                			
            version_defensor = int(request.POST["territorioVersion"].strip())
            num_tropa_def_final = int(request.POST["tropaDefensivaFinal"].strip())
            num_tropa_atac_inic = int(request.POST["tropaAtacanteInic"].strip())
            num_tropa_atac_final = int(request.POST["tropaAtacanteFinal"].strip())
            correcto, resultado = lib_territorio.obtener(partida, territorio_id)
            if correcto:
                territorio_defensor = resultado
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
            correcto, resultado = lib_equipo.obtener(partida, equipo_defensor_id)
            if correcto:                    
                equipo_defensor = resultado
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
            correcto, resultado = lib_marcador.obtener(equipo_defensor)
            if correcto:
                marcador_defensor = resultado                                        
                tropa_defensiva = lib_tropa.obtener_por_territorio(territorio_defensor)[0]
                correcto, resultado = lib_territorio.conquistar_rival(partida, equipo_ganador_id, version_atacante, version_defensor, tropa_defensiva,
                                                                      num_tropa_atac_inic, num_tropa_atac_final,num_tropa_def_final,
                                                                      marcador_atacante, marcador_defensor, puntuacion)
                if correcto:
                    return render_to_response('respuesta.'+formato, {'codigo': 200, 'descripcion': resultado})
                else:
                    return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': resultado}) 
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def equipo_territorio_info(request, partida_id, equipo_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_territorio.obtener(partida, territorio_id)
    if correcto:        
        territorio = resultado
        if lib_territorio.comparar_propietario(territorio, equipo_id):
            tropas = lib_tropa.obtener_por_territorio(territorio)           
            return render_to_response('equipo_territorio_info.' + formato, {'codigo': 200, 'descripcion': 'ok', 'tropas': tropas})
        else:
            msgError = 'version diferente'
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def equipo_territorio_espiar(request, partida_id, equipo_id, territorio_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_territorio.obtener(partida, territorio_id)
    if correcto:        
        territorio = resultado
        correcto, resultado = lib_equipo.modificar_espias(partida, equipo)
        if correcto:
            tropas = lib_tropa.obtener_por_territorio(territorio)
            return render_to_response('equipo_territorio_espiar.' + formato, {'codigo': 200, 'descripcion': resultado, 'tropas': tropas})
        else:
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': resultado})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def equipo_tropa_mover(request, partida_id, equipo_id, tipo_destino, destino_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})    
    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

    if request.method == 'GET':
        tropas = lib_tropa.obtener_por_equipo(equipo)
        return render_to_response('equipo_tropa_mover.' + formato, {'codigo': 200, 'descripcion': 'ok', 'tropas': tropas, 'equipo': equipo})
    elif request.method == 'POST':
        origen_id = int(request.POST["origenId"].strip())                            			
        origen_version = int(request.POST["origenVersion"].strip())
        destino_version = int(request.POST["destinoVersion"].strip())
        tipo_origen = request.POST["tipoOrigen"].strip()
        num_tropa_origen = int(request.POST["numTropaOrigen"].strip())

        if tipo_origen == "equipo":
            origen = equipo
        else:
            correcto, resultado = lib_territorio.obtener(partida, origen_id)
            if correcto:
                origen = lib_tropa.obtener_por_territorio(resultado)[0]
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        if tipo_destino == "equipo":
            destino = equipo
        else:
            correcto, resultado = lib_territorio.obtener(partida, destino_id)
            if correcto:
                destino = lib_tropa.obtener_por_territorio(resultado)[0]
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto,resultado = lib_tropa.mover(partida, tipo_origen, origen_version, destino_version, num_tropa_origen, tipo_destino, origen, destino)
        if correcto:
            return render_to_response('respuesta.'+formato, {'codigo': 200, 'descripcion': resultado})
        else:
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': resultado})

def enfrentamiento_listar(request, partida_id, equipo_id, conquista_id, duelo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    #if formato == 'html':
    #    return HttpResponseRedirect("/conquistadores/usuario/login/")
    
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})     

    conquistas = lib_conquista.listar_por_id(equipo_id, conquista_id)
    duelos = lib_duelo.listar_por_id(equipo_id, duelo_id)
    return render_to_response('enfrentamiento_listar.'+formato, {'codigo': 200, 'descripcion': 'ok', 'conquistas': conquistas, 'duelos': duelos})

        
#FUNCIONES MENSAJES
def mensaje_crear(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    admin = lib_organizador.comprobar_tipo_usuario(request)
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    if request.method == 'GET':
        equipos = lib_equipo.listar_completo(partida)
        return render_to_response('mensaje_crear.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'equipos': equipos})
  
    elif request.method == 'POST':
        tipo_emisor = request.POST["emisor"].strip()
        tipo_recp = request.POST["tipo_receptor"].strip()
        receptor = None
        #Mensaje enviado por un organizador
        if tipo_emisor == "organizador":
            emisor_id = request.session["_auth_user_id"]
            persona = Person.objects.get(id=emisor_id)
            emisor = Organizador.objects.get(usuario=persona)
            #Mensaje para un equipo
            if tipo_recp == "unico":
                equipo_id = request.POST["receptor"].strip()
                correcto, resultado = lib_equipo.obtener(partida, equipo_id)
                if correcto:
                    receptor = resultado
                else:
                    msgError = resultado
                    return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError}) 
        #Mensaje enviado por un equipo
        else:
            correcto, resultado = lib_equipo.obtener(partida, tipo_emisor)
            if correcto:
                emisor = resultado
            else:
                msgError = resultado
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
            
            if tipo_recp == "unico":
                equipo_id = request.POST["receptor"].strip()
                correcto, resultado = lib_equipo.obtener(partida, equipo_id)
                if correcto:
                    receptor = resultado
                else:
                    msgError = resultado
                    return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
                
            elif tipo_recp == "organizador":
                correcto, resultado = lib_organizador.obtener(partida)
                if correcto:
                    receptor = resultado
                else:
                    msgError = resultado
                    return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
            else:
                msgError = "Receptor no valido."
                return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

        texto = request.POST["texto"].strip()  
        mensaje = lib_mensaje.crear(tipo_emisor, tipo_recp, emisor, receptor, texto, partida)
        if formato == 'html':
            url = "/conquistadores/partida/"+str(partida.id)+"/seguimiento/"
            return HttpResponseRedirect(url)
        elif formato == 'json':
            return render_to_response('respuesta.'+formato, {'codigo': 200, 'descripcion': 'ok'})      

def equipo_mensaje_listar(request, partida_id, equipo_id, pet_mensaje):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})     
        
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
        mensajes = lib_mensaje.listar_tipo(partida, equipo_id, pet_mensaje)            
        if len(mensajes) == 0:
            return render_to_response('mensaje_listar.'+ formato, {'codigo': 200, 'descripcion': 'No hay mensajes'})
        else:
            return render_to_response('mensaje_listar.'+ formato, {'codigo': 200, 'descripcion': 'ok', 'mensajes': mensajes})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})          
 
def mensaje_eliminar(request, partida_id, mensaje_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            return HttpResponseRedirect("/conquistadores/usuario/login/")
        
    admin = lib_organizador.comprobar_tipo_usuario(request)
    if admin:
        correcto, resultado = lib_partida.obtener(partida_id)
        if correcto:
            partida = resultado
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        correcto, resultado = lib_mensaje.obtener(partida, mensaje_id)
        if correcto:
            mensaje = resultado
            lib_mensaje.eliminar(mensaje)
            url = "/conquistadores/partida/"+str(partida.id)+"/seguimiento/"
            return HttpResponseRedirect(url)
        else:
            msgError = resultado
            return render_to_response('fallo.'+formato, {'codigo': 500, 'descripcion': msgError})
    else:
        msgError = "Usted no tiene suficiente permisos para realizar esta accion."
        return render_to_response('fallo.'+formato, {'codigo': 200, 'descripcion': msgError})

#FUCNIONES SERVICIO
def servicio_mapa(request, partida_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})
    
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:            
        partida = resultado            
        admin = lib_organizador.comprobar_tipo_usuario(request)
        equipos = lib_equipo.listar_completo(partida)
        territorios = lib_territorio.listar_completo(partida)
        return render_to_response('servicio_mapa.' + formato, {'codigo': 200, 'descripcion': 'ok', 'admin': admin, 'partida': partida, 'equipos':equipos, 'territorios':territorios})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
        
def servicio_mensajeria(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})             

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
        mensajes = lib_mensaje.listar_tipo(partida, equipo_id, 'r')
        if len(mensajes) == 0:
            mensaje_id = -1
            return render_to_response('servicio_mensajeria.'+ formato, {'codigo': 200, 'descripcion': 'No hay mensajes', 'mensaje_id': mensaje_id})
        else:
            mensaje_id = lib_mensaje.obtener_id(mensajes[0])
            return render_to_response('servicio_mensajeria.'+ formato, {'codigo': 200, 'descripcion': 'ok', 'mensaje_id': mensaje_id})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})        

def servicio_localizacion(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})     
        
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
        latitud = float(request.POST["latitud"].strip().replace(",","."))
        longitud = float(request.POST["longitud"].strip().replace(",","."))
        posicion = Point(longitud, latitud, srid=4326)
        lib_equipo.modificar_posicion(posicion, equipo)
        return render_to_response('respuesta.'+formato, {'codigo': 200, 'descripcion': 'ok'})
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})

def servicio_recursos(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})     
        
    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})   

    crecimiento = int(request.POST["crecimiento"].strip())
    puntuacion = int(request.POST["puntuacion"].strip())
    correcto, resultado = lib_equipo.modificar_recursos(partida, equipo_id, crecimiento, puntuacion)
    if correcto:   
        return render_to_response('respuesta.'+formato, {'codigo': 200, 'descripcion': resultado})
    else:
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': resultado})

def servicio_enfrentamiento(request, partida_id, equipo_id):
    correcto, resultado = chequear_formato(request)
    if correcto:
        formato = resultado
    else:
        return render_to_response('fallo.html', {'codigo': 400, 'descripcion': resultado})
    
    if formato == 'html':
        return HttpResponseRedirect("/conquistadores/usuario/login/")

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'codigo': 400, 'descripcion': 'request no valido'})             

    correcto, resultado = lib_partida.obtener(partida_id)
    if correcto:
        partida = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError})
    correcto, resultado = lib_equipo.obtener(partida, equipo_id)
    if correcto:
        equipo = resultado
    else:
        msgError = resultado
        return render_to_response('fallo.'+formato, {'codigo': 400, 'descripcion': msgError}) 

    duelos = lib_duelo.listar_por_equipo(equipo_id) 
    if len(duelos) == 0:
        duelo_id = 0
    else:
        duelo_id = lib_duelo.obtener_id(duelos[0])
    conquistas = lib_conquista.listar_por_equipo(equipo_id)
    if len(conquistas) == 0:
        conquista_id = 0
    else:
        conquista_id = lib_conquista.obtener_id(conquistas[0])

    if conquista_id == 0 and duelo_id == 0:
        return render_to_response('servicio_enfrentamiento.'+ formato, {'codigo': 200, 'descripcion': 'No enfrentamientos', 'dueloId': duelo_id, 'conquistaId': conquista_id})
    else:
        return render_to_response('servicio_enfrentamiento.'+ formato, {'codigo': 200, 'descripcion': 'ok', 'dueloId': duelo_id, 'conquistaId': conquista_id})  
